from typing import TYPE_CHECKING, Dict, List, Union

from loguru import logger
from pydantic import BaseModel

from slark.types.documents.blocks import common as t_block

if TYPE_CHECKING:
    from slark.client.lark import AsyncLark


class Block(BaseModel):
    parent: Union["Block", None] = None
    children: List["Block"] = []
    data: t_block.BlockInfo


class BuildOptions(BaseModel):
    assets_path: str = "./assets/"
    level: int = 0


class PageHelper:
    _client: "AsyncLark"
    block_datas: Dict[str, t_block.BlockInfo]
    page: Block

    def __init__(self, blocks: List[t_block.BlockInfo], client: "AsyncLark") -> "PageHelper":
        """由 BlockInfo 列表构建页面，第一个 BlockInfo 必须为 Page 类型

        Args:
            blocks (List[t_block.BlockInfo]): 文档的 BlockInfo 列表
        """
        assert len(blocks) > 0, "blocks 不能为空"
        assert blocks[0].block_type == t_block.BlockType.PAGE, "第一个 BlockInfo 必须为 Page 类型"

        self.block_datas: Dict[str, t_block.BlockInfo] = {}
        """初始化 Block 数据"""
        for block in blocks:
            self.block_datas[block.block_id] = block

        """构建页面"""
        self.page = self._build_block(blocks[0])
        self._client = client

    def _build_block(self, block_info: t_block.BlockInfo, parent: Block = None) -> Block:
        """构建 Block 对象

        Args:
            block_info (t_block.BlockInfo): Block 的信息
            parent (Block, optional): 父 Block 对象. Defaults to None.

        Returns:
            Block: 构建好的 Block 对象
        """
        block = Block(parent=parent, data=block_info)
        if block_info.children is None:
            return block
        for child_id in block_info.children:
            if child_id not in self.block_datas:
                raise ValueError(f"Block {child_id} 不存在")
            child_block = self._build_block(self.block_datas[child_id], block)
            block.children.append(child_block)
        return block

    async def to_markdown(self, assets_path: str = "./assets/") -> str:
        """将页面转换为 Markdown

        Returns:
            str: Markdown 字符串
        """
        return await self._block_to_markdown(self.page, BuildOptions(assets_path=assets_path))

    async def _text_run_to_markdown(self, text_run: t_block.TextRun):
        md = text_run.content
        if not text_run.text_element_style:
            return md
        if text_run.text_element_style.bold:
            md = f" **{md}** "
        if text_run.text_element_style.italic:
            md = f" *{md}* "
        if text_run.text_element_style.strikethrough:
            md = f" ~~{md}~~ "
        if text_run.text_element_style.underline:
            md = f" _{md}_ "
        if text_run.text_element_style.inline_code:
            md = f" `{md}` "
        if text_run.text_element_style.link:
            from urllib.parse import unquote

            url = unquote(text_run.text_element_style.link.url)
            md = f" [{md}]({url}) "
        """忽略 background_color 和 foreground_color"""
        return md

    async def _equation_to_markdown(self, equation: t_block.Equation):
        return f"$$ {equation.content.strip()} $$"

    async def _mention_doc_to_markdown(self, mention_doc: t_block.MentionDoc):
        return f" [{mention_doc.title}]({mention_doc.url}) "

    async def _text_element_to_markdown(
        self, styles: Union[t_block.TextStyle, None], element: t_block.TextElement
    ):
        if element.text_run:
            return await self._text_run_to_markdown(element.text_run)
        if element.equation:
            return await self._equation_to_markdown(element.equation)
        if element.mention_doc:
            return await self._mention_doc_to_markdown(element.mention_doc)
        return ""

    async def _text_to_markdown(self, text: t_block.Text, type: t_block.BlockType):
        md = "".join(
            [await self._text_element_to_markdown(text.style, element) for element in text.elements]
        )
        """Ingore the children of text block, maybe will be a bug"""
        if not text.style:
            return md
        if text.style.indentation_level == t_block.TextIndentationLevel.ONE_LEVEL_INDENT:
            md = "    " + md
        if type == t_block.BlockType.CODE_BLOCK:
            md = f"```{text.style.language.name.lower()}\n{md}\n```"
        elif type == t_block.BlockType.QUOTE:
            md = f"> {md}"
        elif type == t_block.BlockType.TODO:
            done = "x" if text.style.done else " "
            md = f"- [{done}] {md}"
        elif type.name.startswith("TITLE"):
            md = f"{'#' * int(type.name[-1])} {md}"
        elif type == t_block.BlockType.PAGE:
            md = f"# {md}\n"
        return md

    async def _list_to_markdown(
        self, block: Block, options: BuildOptions, block_type: t_block.BlockType
    ):
        ordered = block_type == t_block.BlockType.ORDERED_LIST
        heading = "1. " if ordered else "- "
        data = block.data.ordered if ordered else block.data.bullet
        assert data is not None, "list 数据不能为空"
        md = await self._text_to_markdown(data, block_type)

        md = "    " * options.level + heading + md
        md = [md]
        for child in block.children:
            options.level += 1
            md.append(await self._block_to_markdown(child, options))
            options.level -= 1
        return "\n".join(md)

    async def _callout_to_markdown(self, callout: Block, options: BuildOptions):
        md = []
        for block in callout.children:
            md.append(f"> {await self._block_to_markdown(block, options=options)}")
        return "\n".join(md)

    async def _divider_to_markdown(self, divider: Block):
        return "---"

    async def _table_to_markdown(self, table: Block, options: BuildOptions):
        assert table.data.table is not None, "table 数据不能为空"
        assert len(table.data.table.cells) == len(table.children), "table 数据长度不匹配"

        def _list_to_row(items: List[str]):
            return "|" + "|".join(items) + "|"

        property = table.data.table.property
        md = []
        for cell in table.children:
            assert cell.data.table_cell is not None, "table_cell 数据不能为空"
            md.append(await self._table_cell_to_markdown(cell, options=options))
        rows = property.row_size
        cols = property.column_size
        header_row = property.header_row
        assert len(md) == rows * cols, "table 数据长度不匹配"
        table_md = []
        for row in range(rows):
            table_md.append(_list_to_row(md[row * cols : (row + 1) * cols]))
        if not header_row:
            table_md.insert(0, _list_to_row(["" for _ in range(cols)]))
        table_md.insert(1, _list_to_row([":---" for _ in range(cols)]))
        return "\n".join(table_md)

    async def _table_cell_to_markdown(self, table_cell: Block, options: BuildOptions):
        md = []
        for block in table_cell.children:
            md.append(await self._block_to_markdown(block, options=options))
        return "\n".join(md)

    async def _quote_container_to_markdown(self, quote_container: Block, options: BuildOptions):
        md = []
        for block in quote_container.children:
            md.append(f"> {await self._block_to_markdown(block, options=options)}")
        return "\n".join(md)

    async def _board_to_markdown(self, board: t_block.Board, options: BuildOptions):
        if options.assets_path is None:
            return ""
        logger.debug(f"Downloading board {board.token} as image to {options.assets_path}")
        path = await self._client.board.download_as_image(board.token, save_dir=options.assets_path)
        return f"![board]({path})"

    async def _image_to_markdown(self, image: t_block.Image, options: BuildOptions):
        if options.assets_path is None:
            return ""
        logger.debug(f"Downloading image {image.token} to {options.assets_path}")
        info = await self._client.assets.download(image.token, save_dir=options.assets_path)
        return f"![{info.filename}]({info.filepath})"

    async def _block_to_markdown(self, block: Block, options: BuildOptions):
        block_type = block.data.block_type
        if block.data.page:
            md = await self._text_to_markdown(block.data.page, block_type)
            md = [
                md,
                *[await self._block_to_markdown(child, options) for child in block.children],
            ]
            return "\n".join(md)
        elif block.data.text:
            return await self._text_to_markdown(block.data.text, block_type)
        elif block.data.heading1:
            return await self._text_to_markdown(block.data.heading1, block_type)
        elif block.data.heading2:
            return await self._text_to_markdown(block.data.heading2, block_type)
        elif block.data.heading3:
            return await self._text_to_markdown(block.data.heading3, block_type)
        elif block.data.heading4:
            return await self._text_to_markdown(block.data.heading4, block_type)
        elif block.data.heading5:
            return await self._text_to_markdown(block.data.heading5, block_type)
        elif block.data.heading6:
            return await self._text_to_markdown(block.data.heading6, block_type)
        elif block.data.heading7:
            return await self._text_to_markdown(block.data.heading7, block_type)
        elif block.data.heading8:
            return await self._text_to_markdown(block.data.heading8, block_type)
        elif block.data.heading9:
            return await self._text_to_markdown(block.data.heading9, block_type)
        elif block.data.bullet:
            return await self._list_to_markdown(block, options=options, block_type=block_type)
        elif block.data.ordered:
            return await self._list_to_markdown(block, options=options, block_type=block_type)
        elif block.data.code:
            return await self._text_to_markdown(block.data.code, block_type)
        elif block.data.equation:
            return await self._text_to_markdown(block.data.equation, block_type)
        elif block.data.quote:
            return await self._text_to_markdown(block.data.quote, block_type)
        elif block.data.todo:
            return await self._text_to_markdown(block.data.todo, block_type)
        elif block.data.callout:
            return await self._callout_to_markdown(block, options=options)
        elif block.data.divider:
            return await self._divider_to_markdown(block)
        elif block.data.table:
            return await self._table_to_markdown(block, options=options)
        elif block.data.quote_container:
            return await self._quote_container_to_markdown(block, options=options)
        elif block.data.board:
            return await self._board_to_markdown(block.data.board, options=options)
        elif block.data.image:
            return await self._image_to_markdown(block.data.image, options=options)
        return ""



from typing import Union

from typing_extensions import Literal

from ..base import BaseElement


class ImageTextElement(BaseElement):
    tag: str = "plain_text"
    content: str


class ImageElement(BaseElement):
    tag: str = "img"
    img_key: str
    alt: ImageTextElement
    title: Union[ImageTextElement, None] = None
    corner_radius: Union[str, None] = None
    scale_type: Literal["crop_center", "crop_top", "fit_horizontal"] = "crop_center"
    """图片的裁剪模式，当 size 字段的比例和图片的比例不一致时会触发裁剪。可取值：

    crop_center：居中裁剪
    crop_top：顶部裁剪
    fit_horizontal：完整展示不裁剪"""

    size: Union[str, None] = None
    """图片尺寸。仅在 scale_type 字段为 crop_center 或 crop_top 时生效。可取值：

    large：大图，尺寸为 160 × 160，适用于多图混排。
    medium：中图，尺寸为 80 × 80，适用于图文混排的封面图。
    small：小图，尺寸为 40 × 40，适用于人员头像。
    tiny：超小图，尺寸为 16 × 16，适用于图标、备注。
    stretch：超大图，适用于高宽比小于 16:9 的图片。
    stretch_without_padding：通栏图，适用于高宽比小于 16:9 的图片，图片的宽度将撑满卡片宽度。
    注意： 卡片 JSON 2.0 结构不再支持 stretch_without_padding 属性。
    你可设置 margin 字段为负数实现通栏效果。如："margin": "4px -12px"。详情参考组件统一支持布局相关能力。
    [1,999]px [1,999]px：自定义图片尺寸，单位为像素，中间用空格分隔。"""

    transparent: bool = False
    """是否为透明底色。默认为 false，即图片为白色底色。"""

    preview: bool = True
    """点击后是否放大图片。

    true：点击图片后，弹出图片查看器放大查看当前点击的图片。
    false：点击图片后，响应卡片本身的交互事件，不弹出图片查看器。
    提示：如果你为卡片配置了跳转链接card_link参数，可将该参数设置为 false，
    后续用户点击卡片上的图片也能响应 card_link 链接跳转。"""


from typing import List, Union

from typing_extensions import TYPE_CHECKING

from ..base import BaseElement
from ..displaying.divider import HlineElement
from ..displaying.image import ImageElement
from ..displaying.image_combination import ImageCombinationElement
from ..displaying.markdown import MDElement
from ..displaying.note import NoteElement
from ..displaying.person import PersonElement
from ..displaying.person_list import PersonListElement
from ..displaying.plain_text import DivElement
from ..interactive.button import ButtonElement
from ..interactive.checker import CheckerElement
from ..interactive.date_picker import DatePickerElement
from ..interactive.input_box import InputBoxElement
from ..interactive.multi_select_person import MultiSelectPersonElement
from ..interactive.multi_select_static import MultiSelectStaticElement
from ..interactive.picker_datetime import PickerDatetimeElement
from ..interactive.picker_time import PickerTimeElement
from ..interactive.select_img import SelectImgElement
from ..interactive.select_person import SelectPersonElement
from ..interactive.select_static import SelectStaticElement

if TYPE_CHECKING:
    from .column import ColumnSetElement

FormElementType = Union[
    MDElement,
    DivElement,
    ImageElement,
    HlineElement,
    "ColumnSetElement",
    ImageCombinationElement,
    PersonElement,
    PersonListElement,
    NoteElement,
    ButtonElement,
    InputBoxElement,
    MultiSelectPersonElement,
    MultiSelectStaticElement,
    SelectPersonElement,
    SelectStaticElement,
    SelectImgElement,
    PickerDatetimeElement,
    PickerTimeElement,
    DatePickerElement,
    CheckerElement,
]


class FormElement(BaseElement):
    tag: str = "form"
    """表单容器的标签。固定值为 form。"""
    name: str
    """表单容器的唯一标识。用于识别用户提交的数据属于哪个表单容器。在同一张卡片内，该字段的值全局唯一。"""
    elements: List[FormElementType]
    """表单容器的子节点。可内嵌其它容器类组件和展示、交互组件，不支持内嵌表格、图表、和表单容器组件。"""



from slark.types._common import BaseModel
from slark.types.response import BaseResponse


class BitableMeta(BaseModel):
    app_token: str
    """多维表格的 app_token"""
    name: str
    """多维表格的名字"""
    revision: int
    """多维表格的版本号（对多维表格进行修改时更新，如新增、删除数据表，修改数据表名等，初始为1，每次更新+1）"""
    is_advanced: bool
    """多维表格是否开启了高级权限。取值包括：
    true：表示开启了高级权限
    false：表示关闭了高级权限"""
    time_zone: str
    """文档时区"""


class UpdateBitableMeta(BaseModel):
    app_token: str


class UpdateBitableMetaResponseData(BaseModel):
    app: UpdateBitableMeta


class GetBitableMetaResponseData(BaseModel):
    app: BitableMeta


class GetBitableMetaResponse(BaseResponse):
    data: GetBitableMetaResponseData


class UpdateBitableMetaResponse(BaseResponse):
    data: UpdateBitableMetaResponseData



from typing import Dict, Iterable, TypedDict, Union

import httpx
from typing_extensions import Literal

from slark._constants import DEFAULT_MAX_RETRIES
from slark.types._common import BaseModel


class RequestOptions(TypedDict):
    headers: dict
    timeout: httpx.Timeout
    params: dict
    max_retries: int
    no_auth: bool
    raw_response: bool
    files: Dict
    data: Dict
    content: Union[bytes, str, Iterable[bytes], Iterable[str]]


class FinalRequestOptions(BaseModel):
    method: Literal["get", "post", "put", "delete", "patch"]
    url: str
    headers: dict = {}
    params: dict = {}
    max_retries: Union[int, None] = DEFAULT_MAX_RETRIES
    timeout: Union[httpx.Timeout, None] = None
    json_data: Union[Dict, None] = None
    files: Union[Dict, None] = None
    data: Union[Dict, None] = None
    content: Union[bytes, str, Iterable[bytes], Iterable[str], None] = None
    no_auth: bool = False
    raw_response: bool = False

    def get_max_retries(self, max_retries: int) -> int:
        return self.max_retries if self.max_retries is not None else max_retries



from typing import Union

from typing_extensions import Literal

from ..base import BaseElement, BaseModel
from ..icon import IconElement
from ..text import TextSize


class MDHrefUrlVal(BaseModel):
    url: str
    pc_url: Union[str, None] = None
    android_url: Union[str, None] = None
    ios_url: Union[str, None] = None


class MDHref(BaseModel):
    urlVal: MDHrefUrlVal


class MDElement(BaseElement):
    tag: str = "markdown"
    content: str
    text_align: Literal["left", "center", "right"] = "left"
    """设置文本内容的对齐方式。可取值有：

    left：左对齐
    center：居中对齐
    right：右对齐"""

    text_size: Union[TextSize, str, None] = TextSize.NORMAL.value
    icon: Union[IconElement, None] = None
    href: Union[MDHref, None] = None


from typing import List, Union

from slark.types._common import BaseModel
from slark.types.response import BaseResponse

from .common import Field


class ListFieldResponseData(BaseModel):
    has_more: bool
    """是否还有更多项"""
    page_token: Union[str, None] = None
    """分页标记，当 has_more 为 true 时，会同时返回新的 page_token，否则不返回 page_token"""
    total: int
    """总数"""
    items: List[Field]
    """字段信息"""


class ListFieldResponse(BaseResponse):
    data: ListFieldResponseData


class CreateFieldResponseData(BaseModel):
    field: Field


class CreateFieldResponse(BaseResponse):
    data: CreateFieldResponseData


class UpdateFieldResponse(BaseResponse):
    data: CreateFieldResponseData


class DeleteFieldResponseData(BaseModel):
    field_id: str
    """field id"""
    deleted: bool
    """删除标记"""


class DeleteFieldResponse(BaseResponse):
    data: DeleteFieldResponseData



from .credentials import CredentailTypes
from .token.request import GetTenantAccessTokenParams
from .token.response import GetTenantAccessTokenResponse
from .token.token import TenantAccessToken, TokenBase, UserAccessToken

__all__ = [
    "GetTenantAccessTokenParams",
    "GetTenantAccessTokenResponse",
    "TenantAccessToken",
    "TokenBase",
    "UserAccessToken",
    "CredentailTypes",
]



from typing import Union

import httpx

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.documents.blocks.request import GetAllBlocksParams
from slark.types.documents.blocks.response import GetAllBlocksResponse


class AsyncBlocks(AsyncAPIResource):
    async def get_all(
        self,
        document_id: str,
        *,
        page_size: Union[int, None] = None,
        page_token: Union[str, None] = None,
        document_revision_id: Union[int, None] = None,
        user_id_type: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetAllBlocksResponse:
        """获取文档的全部 block 信息。
        应用频率限制：单个应用调用频率上限为每秒 5 次，超过该频率限制，接口将返回 HTTP 状态码 400 及错误码 99991400。当请求被限频，应用需要处理限频状态码，并使用指数退避算法或其它一些频控策略降低对 API 的调用速率。

        Args:
            document_id (str): 文档的唯一标识。
            page_size (Union[int, None], optional): 分页大小,默认值/最大值：500. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果. Defaults to None.
            document_revision_id (Union[int, None], optional): 查询的文档版本，-1 表示文档最新版本。文档创建后，版本为 1。若查询的版本为文档最新版本，则需要持有文档的阅读权限；若查询的版本为文档的历史版本，则需要持有文档的编辑权限. Defaults to None.
            user_id_type (Union[str, None], optional): 用户 ID 类型。示例值："open_id"。可选值有：\n
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID\n
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以
                把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？\n
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            GetAllBlocksResponse: 返回的 block 信息。
        """
        return await self._get(
            API_PATH.documents.get_blocks.format(document_id=document_id),
            cast_to=GetAllBlocksResponse,
            options={
                "timeout": timeout,
                "params": GetAllBlocksParams(
                    page_size=page_size,
                    page_token=page_token,
                    document_revision_id=document_revision_id,
                    user_id_type=user_id_type,
                ).model_dump(),
            },
        )


class Blocks(APIResource):
    def get_all(
        self,
        document_id: str,
        *,
        page_size: Union[int, None] = None,
        page_token: Union[str, None] = None,
        document_revision_id: Union[int, None] = None,
        user_id_type: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetAllBlocksResponse:
        """获取文档的全部 block 信息。
        应用频率限制：单个应用调用频率上限为每秒 5 次，超过该频率限制，接口将返回 HTTP 状态码 400 及错误码 99991400。当请求被限频，应用需要处理限频状态码，并使用指数退避算法或其它一些频控策略降低对 API 的调用速率。

        Args:
            document_id (str): 文档的唯一标识。
            page_size (Union[int, None], optional): 分页大小,默认值/最大值：500. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果. Defaults to None.
            document_revision_id (Union[int, None], optional): 查询的文档版本，-1 表示文档最新版本。文档创建后，版本为 1。若查询的版本为文档最新版本，则需要持有文档的阅读权限；若查询的版本为文档的历史版本，则需要持有文档的编辑权限. Defaults to None.
            user_id_type (Union[str, None], optional): 用户 ID 类型。示例值："open_id"。可选值有：\n
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID\n
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以
                把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？\n
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            GetAllBlocksResponse: 返回的 block 信息。
        """
        return self._get(
            API_PATH.documents.get_blocks.format(document_id=document_id),
            cast_to=GetAllBlocksResponse,
            options={
                "timeout": timeout,
                "params": GetAllBlocksParams(
                    page_size=page_size,
                    page_token=page_token,
                    document_revision_id=document_revision_id,
                    user_id_type=user_id_type,
                ).model_dump(),
            },
        )



from typing import Tuple

from slark.types._common import BaseModel


def row_column_to_excel(row: int, column: int) -> str:
    """Convert row and column to excel format.
    For example, 0,0 => A1, 1,0 => A2, 0,26 => AA1"""
    excel = ""
    while column >= 0:
        excel = chr(65 + column % 26) + excel
        column = column // 26 - 1
    return excel + str(row + 1)


def excel_to_row_column(excel: str) -> Tuple[int, int]:
    """Convert excel to row and column.
    For example, A1 => 0,0, A2 => 1,0, AA1 => 0,26"""
    column = 0
    for c in excel:
        if c.isalpha():
            column = column * 26 + ord(c) - 65
        else:
            row = int(excel[len(str(column)) :]) - 1
            return row, column
    return row, column


class Range(BaseModel):
    start_row: int = 0
    start_col: int = 0
    rows: int
    cols: int
    sheet_id: str

    @property
    def excel(self) -> str:
        start_pos = row_column_to_excel(self.start_row, self.start_col)
        end_pos = row_column_to_excel(
            self.start_row + self.rows - 1, self.start_col + self.cols - 1
        )
        return f"{self.sheet_id}!{start_pos}:{end_pos}"

    def intersect(self, other: "Range") -> "Range":
        if self.sheet_id != other.sheet_id:
            raise ValueError("Cannot intersect ranges from different sheets")
        if (
            self.start_row > other.start_row + other.rows - 1
            or other.start_row > self.start_row + self.rows - 1
        ):
            raise ValueError("Ranges do not intersect")
        if (
            self.start_col > other.start_col + other.cols - 1
            or other.start_col > self.start_col + self.cols - 1
        ):
            raise ValueError("Ranges do not intersect")
        start_row = max(self.start_row, other.start_row)
        start_col = max(self.start_col, other.start_col)
        end_row = min(self.start_row + self.rows - 1, other.start_row + other.rows - 1)
        end_col = min(self.start_col + self.cols - 1, other.start_col + other.cols - 1)
        return Range(
            start_row=start_row,
            start_col=start_col,
            rows=end_row - start_row + 1,
            cols=end_col - start_col + 1,
            sheet_id=self.sheet_id,
        )



from typing import Any, Union

from pydantic import BaseModel, ConfigDict


class BaseResponse(BaseModel):
    code: int
    """错误码，非 0 表示失败"""
    msg: str
    """错误描述"""
    data: Union[Any, None] = None
    """返回数据"""
    error: Union[Any, None] = None
    """错误信息"""

    model_config = ConfigDict(extra="allow")



from typing import Dict, List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel

from .common import FieldValueType


class CreateRecordParams(BaseModel):
    user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None
    client_token: Union[str, None] = None
    """格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。

    示例值："fe599b60-450f-46ff-b2ef-9f6675625b97"""


class CreateRecordBody(BaseModel):
    """用于创建多维表格记录的请求体
    Example:
    ```
    {
        "fields": {
            "多行文本": "多行文本内容",
            "条码": "+$$3170930509104X512356",
            "数字": 100,
            "货币": 3,
            "评分": 3,
            "进度": 0.25,
            "单选": "选项1",
            "多选": ["选项1", "选项2"],
            "日期": 1674206443000,
            "复选框": true,
            "人员": [
                {"id": "ou_2910013f1e6456f16a0ce75ede950a0a"},
                {"id": "ou_e04138c9633dd0d2ea166d79f548ab5d"},
            ],
            "群组": [{"id": "oc_cd07f55f14d6f4a4f1b51504e7e97f48"}],
            "电话号码": "13026162666",
            "超链接": {"text": "飞书多维表格官网", "link": "https://www.feishu.cn/product/base"},
            "附件": [
                {"file_token": "DRiFbwaKsoZaLax4WKZbEGCccoe"},
                {"file_token": "BZk3bL1Enoy4pzxaPL9bNeKqcLe"},
                {"file_token": "EmL4bhjFFovrt9xZgaSbjJk9c1b"},
                {"file_token": "Vl3FbVkvnowlgpxpqsAbBrtFcrd"},
            ],
            "单向关联": ["recHTLvO7x", "recbS8zb2m"],
            "双向关联": ["recHTLvO7x", "recbS8zb2m"],
            "地理位置": "116.397755,39.903179",
        }
    }
    ```
    """

    fields: Dict[str, FieldValueType]


class UpdateRecordParams(BaseModel):
    user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None


class UpdateRecordBody(BaseModel):
    fields: Dict[str, FieldValueType]


class SearchRecordParams(BaseModel):
    user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None
    """用户 ID 类型

    示例值："open_id"

    可选值有：

    open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID
    union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？
    user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？
    默认值：open_id

    当值为 user_id，字段权限要求：

    获取用户 user ID
    仅自建应用"""

    page_token: Union[str, None] = None
    """分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果

    示例值："eVQrYzJBNDNONlk4VFZBZVlSdzlKdFJ4bVVHVExENDNKVHoxaVdiVnViQT0="."""

    page_size: Union[int, None] = None
    """分页大小。最大值为 500

    示例值：10

    默认值：20"""


class SearchRecordSort(BaseModel):
    field_name: str
    """字段名称

    示例值："多行文本"

    数据校验规则：

    长度范围：0 字符 ～ 1000 字符"""

    desc: Union[bool, None] = None
    """是否倒序排序

    示例值：true

    默认值：false"""


class SearchRecordFilterCondition(BaseModel):
    field_name: str
    """筛选条件的左值，值为字段的名称

    示例值："字段1"

    数据校验规则：

    长度范围：0 字符 ～ 1000 字符"""

    operator: Literal[
        "is",
        "isNot",
        "contains",
        "doesNotContain",
        "isEmpty",
        "isNotEmpty",
        "isGreater",
        "isGreaterEqual",
        "isLess",
        "isLessEqual",
        "like",
        "in",
    ]
    """条件运算符

    示例值："is"

    可选值有：

    is：等于
    isNot：不等于
    contains：包含
    doesNotContain：不包含
    isEmpty：为空
    isNotEmpty：不为空
    isGreater：大于
    isGreaterEqual：大于等于
    isLess：小于
    isLessEqual：小于等于
    like：LIKE 运算符。暂未支持
    in：IN 运算符。暂未支持"""

    value: List[str]
    """目标值

    [目标值填写指南](https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/record-filter-guide)

    示例值：["文本内容"]

    数据校验规则：

    长度范围：0 ～ 10"""


class SearchRecordFilter(BaseModel):
    conjunction: Union[Literal["and", "or"], None] = None
    """条件逻辑连接词

    示例值："and"

    可选值有：

    and：满足全部条件
    or：满足任一条件
    数据校验规则：

    长度范围：0 字符 ～ 10 字符"""
    conditions: List[SearchRecordFilterCondition]


class SearchRecordBody(BaseModel):
    """用于搜索多维表格记录的请求体
        Example:
        ```
        {
        "view_id":"vewqhz51lk",
        "field_names":[
            "字段1",
            "字段2"
        ],
        "sort":[
            {
                "field_name":"多行文本",
                "desc":true
            }
        ],
        "filter":{
            "conjunction":"and",
            "conditions":[
                {
                    "field_name":"字段1",
                    "operator":"is",
                    "value":[
                        "文本内容"
                    ]
                }
            ]
        },
        "automatic_fields":false
    }```"""

    view_id: Union[str, None] = None
    """视图的唯一标识符，获取指定视图下的记录view_id 参数说明

    注意：当 filter 参数 或 sort 参数不为空时，请求视为对数据表中的全部数据做条件过滤，指定的view_id 会被忽略。

    示例值："vewqhz51lk"

    数据校验规则：

    长度范围：0 字符 ～ 50 字符"""

    field_names: Union[List[str], None] = None
    """字段名称，用于指定本次查询返回记录中包含的字段

    示例值：["字段1","字段2"]

    数据校验规则：

    长度范围：0 ～ 200"""

    sort: Union[List[SearchRecordSort], None] = None
    """排序条件

    数据校验规则：

    长度范围：0 ～ 100"""

    filter: Union[SearchRecordFilter, None] = None
    """筛选条件"""

    automatic_fields: Union[bool, None] = None
    """控制是否返回自动计算的字段, true 表示返回

    示例值：false"""


class BatchCreateRecordBody(BaseModel):
    records: List[CreateRecordBody]


class BatchUpdateRecord(BaseModel):
    record_id: str
    """一条记录的唯一标识 ID。参考record_id 参数说明 获取 record_id。该参数必填，请忽略左侧必填列的否

    示例值："recqwIwhc6"."""

    fields: Dict[str, FieldValueType]
    """数据表的字段，即数据表的列

    当前接口支持的字段类型请参考接入指南

    不同类型字段的数据结构请参考数据结构概述

    示例值：{"多行文本":"HelloWorld"}"""


class BatchUpdateRecordBody(BaseModel):
    """用于批量更新多维表格记录的请求体
        Example:
        ```{
        "records": [
            {
                "record_id": "reclAqylTN",
                "fields": {
                    "索引": "索引列多行文本类型",
                    "多行文本": "多行文本内容",
                    "数字": 100,
                    "单选": "选项3",
                    "多选": [
                        "选项1",
                        "选项2"
                    ],
                    "日期": 1674206443000,
                    "条码": "qawqe",
                    "复选框": true,
                    "人员": [
                        {
                            "id": "ou_2910013f1e6456f16a0ce75ede950a0a"
                        },
                        {
                            "id": "ou_e04138c9633dd0d2ea166d79f548ab5d"
                        }
                    ],
                    "群组": [
                        {
                            "id": "oc_cd07f55f14d6f4a4f1b51504e7e97f48"
                        }
                    ],
                    "电话号码": "13026162666",
                    "超链接": {
                        "text": "飞书多维表格官网",
                        "link": "https://www.feishu.cn/product/base"
                    },
                    "附件": [
                        {
                            "file_token": "Vl3FbVkvnowlgpxpqsAbBrtFcrd"
                        }
                    ],
                    "单向关联": [
                        "recHTLvO7x",
                        "recbS8zb2m"
                    ],
                    "双向关联": [
                        "recHTLvO7x",
                        "recbS8zb2m"
                    ],
                    "地理位置": "116.397755,39.903179",
                    "评分": 3,
                    "货币": 3,
                    "进度": 0.25
                }
            }
        ]
    }```"""

    records: List[BatchUpdateRecord]


class BatchGetRecordBody(BaseModel):
    """用于批量获取多维表格记录的请求体
    Example:
    ```
    {
        "record_ids": ["recyOaMB2F", "rec111111", "recyOaMB2F"],
        "user_id_type": "open_id",
        "with_shared_url": true,
        "automatic_fields": true,
    }```
    """

    record_ids: List[str]
    """记录 ID 列表。调用列出记录获取。

    示例值：["recyO3299N"]

    数据校验规则：

    长度范围：1 ～ 100"""

    user_id_type: Union[str, None] = None
    """此次调用中使用的用户 id 的类型

    示例值："open_id"

    可选值有：

    user_id：以user_id来识别用户
    union_id：以union_id来识别用户
    open_id：以open_id来识别用户"""

    with_shared_url: Union[bool, None] = None
    """是否返回记录的分享链接。可选值：

    true：返回分享链接
    false：不返回分享链接
    默认值：false

    示例值：true"""

    automatic_fields: Union[bool, None] = None
    """是否返回自动计算的字段。可选值：

    true：返回自动计算的字段
    false：不返回自动计算的字段
    默认值：false

    示例值：true"""


class BatchDeleteRecordBody(BaseModel):
    records: List[str]
    """待删除的记录的id

    示例值：["recyO3299N"]"""



from typing import Union

from typing_extensions import Literal

from slark.types._common import BaseModel


class GetRawContentParams(BaseModel):
    lang: Union[Literal[0, 1, 2], None] = None
    """指定返回的 MentionUser 即 @用户 的语言。\
        示例值：0。\
        可选值有：\n
            0：该用户的默认名称。如：@张敏。\n
            1：该用户的英文名称。如：@Min Zhang。\n
            2：暂不支持该枚举，使用时返回该用户的默认名称。\n
        默认值：0"""




from typing import Dict, List, Union

from typing_extensions import Literal

from .._common import BaseModel
from .container.collapsible import CollapsibleElement
from .container.column import ColumnSetElement
from .container.form import FormElement
from .container.interactive_container import InteractiveContainerElement
from .displaying.chart import ChartElement
from .displaying.divider import HlineElement
from .displaying.header import CardHeader, CardHeaderTextTag, CardTemplate
from .displaying.image import ImageElement
from .displaying.image_combination import ImageCombinationElement
from .displaying.markdown import MDElement
from .displaying.note import NoteElement
from .displaying.person import PersonElement
from .displaying.person_list import PersonListElement
from .displaying.plain_text import DivElement, PlainTextElement
from .displaying.table import TableElement
from .icon import IconElement
from .interactive.button import ButtonElement
from .interactive.checker import CheckerElement
from .interactive.date_picker import DatePickerElement
from .interactive.input_box import InputBoxElement
from .interactive.multi_select_person import MultiSelectPersonElement
from .interactive.multi_select_static import MultiSelectStaticElement
from .interactive.overflow import OverflowElement
from .interactive.picker_datetime import PickerDatetimeElement
from .interactive.picker_time import PickerTimeElement
from .interactive.select_img import SelectImgElement
from .interactive.select_person import SelectPersonElement
from .interactive.select_static import SelectStaticElement

"""飞书卡片文档链接 https://open.feishu.cn/document/uAjLw4CM/ukzMukzMukzM/feishu-cards/card-json-structure"""


class CardConfigStyleTextSize(BaseModel):
    default: str = "medium"
    """在无法差异化配置字号的旧版飞书客户端上，生效的字号属性。选填。"""
    pc: str = "medium"
    """桌面端的字号。"""
    mobile: str = "medium"
    """移动端的字号。"""


class CardConfigStyleColor(BaseModel):
    light_mode: str = "rgba(5,157,178,0.52)"
    """浅色主题下的自定义颜色语法"""
    dark_mode: str = "rgba(78,23,108,0.49)"
    """深色主题下的自定义颜色语法"""


class CardConfigStyle(BaseModel):
    text_size: Dict[str, CardConfigStyleTextSize] = {"cus-0": CardConfigStyleTextSize()}
    """分别为移动端和桌面端添加自定义字号，同时添加兜底字号。用于在组件 JSON 中设置字号属性。支持添加多个自定义字号对象。"""
    color: Dict[str, CardConfigStyleColor] = {"cus-0": CardConfigStyleColor()}
    """分别为飞书客户端浅色主题和深色主题添加 RGBA 语法。用于在组件 JSON 中设置颜色属性。支持添加多个自定义颜色对象。"""


class CardConfig(BaseModel):
    enable_forward: Union[bool, None] = None
    """是否支持转发卡片。默认值 true"""
    update_multi: Union[bool, None] = None
    """是否为共享卡片。为 true 时即更新卡片的内容对所有收到这张卡片的人员可见。默认值 false。"""
    width_mode: Union[Literal["fill", "compact", "default"], None] = None
    """卡片宽度模式。支持 "compact"（紧凑宽度 400px）模式、"fill"（撑满聊天窗口宽度）模式和 "default" 默认模式(宽度上限为 600px)。"""
    compact_width: Union[bool, None] = None
    """已废弃字段。是否为紧凑型卡片宽度。与 width_mode 属性同时设置时，width_mode 将生效。"""
    use_custom_translation: Union[bool, None] = None
    """是否使用自定义翻译数据。默认值 false。为 true 时则在用户点击消息翻译后，使用 i18n 对应的目标语种作为翻译结果。若 i18n 取不到，则使用当前内容请求翻译，不使用自定义翻译数据。"""
    enable_forward_interaction: Union[bool, None] = None
    """转发的卡片是否仍然支持回传交互。默认值 false。"""
    style: Union[CardConfigStyle, None] = None


class CardLink(BaseModel):
    url: str
    android_url: Union[str, None] = None
    ios_url: Union[str, None] = None
    pc_url: Union[str, None] = None


ElementType = Union[
    ChartElement,
    HlineElement,
    DivElement,
    ImageCombinationElement,
    ImageElement,
    MDElement,
    NoteElement,
    PersonElement,
    PersonListElement,
    PlainTextElement,
    TableElement,
    ColumnSetElement,
    FormElement,
    InteractiveContainerElement,
    CollapsibleElement,
    ButtonElement,
    CheckerElement,
    DatePickerElement,
    InputBoxElement,
    MultiSelectPersonElement,
    MultiSelectStaticElement,
    OverflowElement,
    PickerDatetimeElement,
    PickerTimeElement,
    SelectImgElement,
    SelectPersonElement,
    SelectStaticElement,
]


class I18nBody(BaseModel):
    zh_cn: Union[List[ElementType], None] = None
    en_us: Union[List[ElementType], None] = None
    ja_jp: Union[List[ElementType], None] = None
    zh_hk: Union[List[ElementType], None] = None
    zh_tw: Union[List[ElementType], None] = None
    id_id: Union[List[ElementType], None] = None
    vi_vn: Union[List[ElementType], None] = None
    th_th: Union[List[ElementType], None] = None
    pt_br: Union[List[ElementType], None] = None
    es_es: Union[List[ElementType], None] = None
    ko_kr: Union[List[ElementType], None] = None
    de_de: Union[List[ElementType], None] = None
    fr_fr: Union[List[ElementType], None] = None
    it_it: Union[List[ElementType], None] = None
    ru_ru: Union[List[ElementType], None] = None
    ms_my: Union[List[ElementType], None] = None


class I18nHeaderElement(BaseModel):
    title: Union[PlainTextElement, None] = None
    subtitle: Union[PlainTextElement, None] = None
    template: Union[CardTemplate, None] = None
    ud_icon: Union[IconElement, None] = None
    text_tag_list: Union[List[CardHeaderTextTag], None] = None


class I18nHeader(BaseModel):
    zh_cn: Union[I18nHeaderElement, None] = None
    en_us: Union[I18nHeaderElement, None] = None
    ja_jp: Union[I18nHeaderElement, None] = None
    zh_hk: Union[I18nHeaderElement, None] = None
    zh_tw: Union[I18nHeaderElement, None] = None
    id_id: Union[I18nHeaderElement, None] = None
    vi_vn: Union[I18nHeaderElement, None] = None
    th_th: Union[I18nHeaderElement, None] = None
    pt_br: Union[I18nHeaderElement, None] = None
    es_es: Union[I18nHeaderElement, None] = None
    ko_kr: Union[I18nHeaderElement, None] = None
    de_de: Union[I18nHeaderElement, None] = None
    fr_fr: Union[I18nHeaderElement, None] = None
    it_it: Union[I18nHeaderElement, None] = None
    ru_ru: Union[I18nHeaderElement, None] = None
    ms_my: Union[I18nHeaderElement, None] = None


class InteractiveCard(BaseModel):
    config: Union[CardConfig, None] = None
    """config 用于配置卡片的全局行为，包括是否允许被转发、是否为共享卡片等。"""
    card_link: Union[CardLink, None] = None
    """card_link 字段用于指定卡片整体的点击跳转链接。你可以配置一个默认链接，也可以分别为 PC 端、Android 端、iOS 端配置不同的跳转链接。"""
    i18n_elements: Union[I18nBody, None] = None
    """飞书卡片支持多语言设置。设置多语言后，卡片将根据用户的飞书客户端语言，自动展示对应语言的卡片内容，满足国际化业务需求。详情参考配置卡片多语言。"""
    header: Union[CardHeader, None] = None
    """标题组件 JSON 代码。详细字段说明参考标题组件。"""
    i18n_header: Union[I18nHeader, None] = None
    """多语言表头"""
    elements: Union[List[ElementType], None] = None
    """在 elements 字段中添加各个组件的 JSON 数据，组件将按数组顺序纵向流式排列。了解各个组件，参考组件概述。"""
    fallback: Union[Dict, None] = None
    """fallback 用于为卡片添加全局降级规则。触发降级时，卡片将全局展示“请升级客户端至最新版本后查看”占位图。
    注意：该字段要求飞书客户端的版本为 V7.7 及以上。"""



import enum
from typing import List, Union

import pydantic
from typing_extensions import Literal


class BlockType(enum.Enum):
    """Block 类型

    可选值有：

    1：页面 Block

    2：文本 Block

    3：标题 1 Block

    4：标题 2 Block

    5：标题 3 Block

    6：标题 4 Block

    7：标题 5 Block

    8：标题 6 Block

    9：标题 7 Block

    10：标题 8 Block

    11：标题 9 Block

    12：无序列表 Block

    13：有序列表 Block

    14：代码块 Block

    15：引用 Block

    17：待办事项 Block

    18：多维表格 Block

    19：高亮块 Block

    20：会话卡片 Block

    21：流程图 & UML Block

    22：分割线 Block

    23：文件 Block

    24：分栏 Block

    25：分栏列 Block

    26：内嵌网页 Block

    27：图片 Block

    28：开放平台小组件 Block

    29：思维笔记 Block

    30：电子表格 Block

    31：表格 Block。了解如何在文档中插入表格，参考文档常见问题-如何插入表格并往单元格填充内容。

    32：表格单元格 Block

    33：视图 Block

    34：引用容器 Block

    35：任务 Block

    36：OKR Block

    37：OKR Objective Block

    38：OKR Key Result Block

    39：OKR 进展 Block

    40：文档小组件 Block

    41：Jira 问题 Block

    42：Wiki 子目录 Block

    43：画板 Block

    44：议程 Block

    45：议程项 Block

    46：议程项标题 Block

    47：议程项内容 Block

    48：链接预览 Block

    999：未支持 Block"""

    PAGE = 1
    TEXT = 2
    TITLE1 = 3
    TITLE2 = 4
    TITLE3 = 5
    TITLE4 = 6
    TITLE5 = 7
    TITLE6 = 8
    TITLE7 = 9
    TITLE8 = 10
    TITLE9 = 11
    UNORDERED_LIST = 12
    ORDERED_LIST = 13
    CODE_BLOCK = 14
    QUOTE = 15
    TODO = 17
    MULTI_DIMENSION_TABLE = 18
    HIGHLIGHT = 19
    CONVERSATION_CARD = 20
    FLOWCHART_UML = 21
    SEPARATOR = 22
    FILE = 23
    COLUMN = 24
    COLUMN_ITEM = 25
    EMBEDDED_WEBPAGE = 26
    IMAGE = 27
    OPEN_PLATFORM_WIDGET = 28
    MIND_MAP = 29
    SPREADSHEET = 30
    TABLE = 31
    TABLE_CELL = 32
    VIEW = 33
    REFERENCE_CONTAINER = 34
    TASK = 35
    OKR = 36
    OKR_OBJECTIVE = 37
    OKR_KEY_RESULT = 38
    OKR_PROGRESS = 39
    DOCUMENT_WIDGET = 40
    JIRA_ISSUE = 41
    WIKI_SUBDIRECTORY = 42
    CANVAS = 43
    AGENDA = 44
    AGENDA_ITEM = 45
    AGENDA_ITEM_TITLE = 46
    AGENDA_ITEM_CONTENT = 47
    LINK_PREVIEW = 48
    UNSUPPORTED = 999


class TextStyleAlign(enum.Enum):
    LEFT = 1
    CENTER = 2
    RIGHT = 3


class CodeBlockLanguage(enum.Enum):
    """代码块的语言类型。仅支持对 Code 块进行修改
    可选值有：

    1：PlainText

    2：ABAP

    3：Ada

    4：Apache

    5：Apex

    6：Assembly Language

    7：Bash

    8：CSharp

    9：C++

    10：C

    11：COBOL

    12：CSS

    13：CoffeeScript

    14：D

    15：Dart

    16：Delphi

    17：Django

    18：Dockerfile

    19：Erlang

    20：Fortran

    21：FoxPro

    22：Go

    23：Groovy

    24：HTML

    25：HTMLBars

    26：HTTP

    27：Haskell

    28：JSON

    29：Java

    30：JavaScript

    31：Julia

    32：Kotlin

    33：LateX

    34：Lisp

    35：Logo

    36：Lua

    37：MATLAB

    38：Makefile

    39：Markdown

    40：Nginx

    41：Objective-C

    42：OpenEdgeABL

    43：PHP

    44：Perl

    45：PostScript

    46：Power Shell

    47：Prolog

    48：ProtoBuf

    49：Python

    50：R

    51：RPG

    52：Ruby

    53：Rust

    54：SAS

    55：SCSS

    56：SQL

    57：Scala

    58：Scheme

    59：Scratch

    60：Shell

    61：Swift

    62：Thrift

    63：TypeScript

    64：VBScript

    65：Visual Basic

    66：XML

    67：YAML

    68：CMake

    69：Diff

    70：Gherkin

    71：GraphQL

    72：OpenGL Shading Language

    73：Properties

    74：Solidity

    75：TOML"""

    PLAIN_TEXT = 1
    ABAP = 2
    ADA = 3
    APACHE = 4
    APEX = 5
    ASSEMBLY_LANGUAGE = 6
    BASH = 7
    CSHARP = 8
    CPLUSPLUS = 9
    C = 10
    COBOL = 11
    CSS = 12
    COFFEESCRIPT = 13
    D = 14
    DART = 15
    DELPHI = 16
    DJANGO = 17
    DOCKERFILE = 18
    ERLANG = 19
    FORTRAN = 20
    FOXPRO = 21
    GO = 22
    GROOVY = 23
    HTML = 24
    HTMLBARS = 25
    HTTP = 26
    HASKELL = 27
    JSON = 28
    JAVA = 29
    JAVASCRIPT = 30
    JULIA = 31
    KOTLIN = 32
    LATEX = 33
    LISP = 34
    LOGO = 35
    LUA = 36
    MATLAB = 37
    MAKEFILE = 38
    MARKDOWN = 39
    NGINX = 40
    OBJECTIVE_C = 41
    OPENEDGEABL = 42
    PHP = 43
    PERL = 44
    POSTSCRIPT = 45
    POWER_SHELL = 46
    PROLOG = 47
    PROTOBUF = 48
    PYTHON = 49
    R = 50
    RPG = 51
    RUBY = 52
    RUST = 53
    SAS = 54
    SCSS = 55
    SQL = 56
    SCALA = 57
    SCHEME = 58
    SCRATCH = 59
    SHELL = 60
    SWIFT = 61
    THRIFT = 62
    TYPESCRIPT = 63
    VBSCRIPT = 64
    VISUAL_BASIC = 65
    XML = 66
    YAML = 67
    CMAKE = 68
    DIFF = 69
    GHERKIN = 70
    GRAPHQL = 71
    OPENGL_SHADING_LANGUAGE = 72
    PROPERTIES = 73
    SOLIDITY = 74
    TOML = 75


class TextBackgroundColor(enum.Enum):
    """块的背景色

    可选值有：

    LightGrayBackground：浅灰色

    LightRedBackground：浅红色

    LightOrangeBackground：浅橙色

    LightYellowBackground：浅黄色

    LightGreenBackground：浅绿色

    LightBlueBackground：浅蓝色

    LightPurpleBackground：浅紫色

    PaleGrayBackground：淡灰色

    DarkGrayBackground：深灰色

    DarkRedBackground：深红色

    DarkOrangeBackground：深橙色

    DarkYellowBackground：深黄色

    DarkGreenBackground：深绿色

    DarkBlueBackground：深蓝色

    DarkPurpleBackground：深紫色

    """

    LIGHT_GRAY = "LightGrayBackground"
    LIGHT_RED = "LightRedBackground"
    LIGHT_ORANGE = "LightOrangeBackground"
    LIGHT_YELLOW = "LightYellowBackground"
    LIGHT_GREEN = "LightGreenBackground"
    LIGHT_BLUE = "LightBlueBackground"
    LIGHT_PURPLE = "LightPurpleBackground"
    PALE_GRAY = "PaleGrayBackground"
    DARK_GRAY = "DarkGrayBackground"
    DARK_RED = "DarkRedBackground"
    DARK_ORANGE = "DarkOrangeBackground"
    DARK_YELLOW = "DarkYellowBackground"
    DARK_GREEN = "DarkGreenBackground"
    DARK_BLUE = "DarkBlueBackground"
    DARK_PURPLE = "DarkPurpleBackground"


class TextElementBackgroundColor(enum.Enum):
    """背景色

    可选值有：

    1：浅粉红色

    2：浅橙色

    3：浅黄色

    4：浅绿色

    5：浅蓝色

    6：浅紫色

    7：浅灰色

    8：暗粉红色

    9：暗橙色

    10：暗黄色

    11：暗绿色

    12：暗蓝色

    13：暗紫色

    14：暗灰色

    15：暗银灰色"""

    LIGHT_PINK = 1
    LIGHT_ORANGE = 2
    LIGHT_YELLOW = 3
    LIGHT_GREEN = 4
    LIGHT_BLUE = 5
    LIGHT_PURPLE = 6
    LIGHT_GRAY = 7
    DARK_PINK = 8
    DARK_ORANGE = 9
    DARK_YELLOW = 10
    DARK_GREEN = 11
    DARK_BLUE = 12
    DARK_PURPLE = 13
    DARK_GRAY = 14
    DARK_SILVER_GRAY = 15


class TextElementTextColor(enum.Enum):
    """
    字体颜色

    可选值有：

    1：粉红色

    2：橙色

    3：黄色

    4：绿色

    5：蓝色

    6：紫色

    7：灰色"""

    PINK = 1
    ORANGE = 2
    YELLOW = 3
    GREEN = 4
    BLUE = 5
    PURPLE = 6
    GRAY = 7


class BorderColor(enum.Enum):
    """
    边框色

    可选值有：

    1：红色

    2：橙色

    3：黄色

    4：绿色

    5：蓝色

    6：紫色

    7：灰色"""

    RED = 1
    ORANGE = 2
    YELLOW = 3
    GREEN = 4
    BLUE = 5
    PURPLE = 6
    GRAY = 7


class TextIndentationLevel(enum.Enum):
    NO_INDENT = "NoIndent"
    ONE_LEVEL_INDENT = "OneLevelIndent"


class BaseModel(pydantic.BaseModel):
    model_config = pydantic.ConfigDict()


class TextStyle(BaseModel):
    align: Union[TextStyleAlign, None] = None
    """对齐方式"""
    done: Union[bool, None] = None
    """todo 的完成状态。支持对 Todo 块进行修改"""
    folded: Union[bool, None] = None
    """文本的折叠状态。支持对 Heading1~9、和有子块的 Text、Ordered、Bullet 和 Todo 块进行修改"""
    language: Union[CodeBlockLanguage, None] = None
    """代码块的语言类型。仅支持对 Code 块进行修改"""
    wrap: Union[bool, None] = None
    """代码块是否自动换行。支持对 Code 块进行修改"""
    background_color: Union[TextBackgroundColor, None] = None
    """块的背景色"""
    indentation_level: Union[TextIndentationLevel, None] = None
    """首行缩进级别"""


class DocType(enum.Enum):
    """云文档类型

    可选值有：

    1：Doc

    3：Sheet

    8：Bitable

    11：MindNote

    12：File

    15：Slide

    16：Wiki

    22：Docx"""

    DOC = 1
    SHEET = 3
    BITABLE = 8
    MINDNOTE = 11
    FILE = 12
    SLIDE = 15
    WIKI = 16
    DOCX = 22


class DiagramType(enum.Enum):
    FLOWCHART = 1
    UML = 2


class ViewType(enum.Enum):
    CARD = 1
    PREVIEW = 2


class IframeType(enum.Enum):
    """
    iframe 类型

    可选值有：

    1：哔哩哔哩

    2：西瓜视频

    3：优酷

    4：Airtable

    5：百度地图"""

    BILIBILI = 1
    XIGUA_VIDEO = 2
    YOUKU = 3
    AIRTABLE = 4
    BAIDU_MAP = 5


class TextElementLink(BaseModel):
    url: str


class TextElementStyle(BaseModel):
    bold: Union[bool, None] = None
    """加粗"""
    italic: Union[bool, None] = None
    """斜体"""
    strikethrough: Union[bool, None] = None
    """删除线"""
    underline: Union[bool, None] = None
    """下划线"""
    inline_code: Union[bool, None] = None
    """行内代码"""
    backgroud_color: Union[TextElementBackgroundColor, None] = None
    """背景色"""
    text_color: Union[TextElementTextColor, None] = None
    """字体颜色"""
    link: Union[TextElementLink, None] = None
    """链接"""
    comment_ids: Union[List[str], None] = None
    """评论 ID 列表"""


class TextRun(BaseModel):
    content: str
    """文本内容"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本样式"""


class MentionUser(BaseModel):
    user_id: str
    """用户 OpenID"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本样式"""


class MentionDoc(BaseModel):
    token: str
    """文档 Token"""
    obj_type: Union[DocType, None] = None
    """云文档类型"""
    url: str
    """云文档链接（需要 url_encode)"""
    title: str
    """文档标题，只读属性"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本局部样式"""


class Reminder(BaseModel):
    create_user_id: str
    """创建者用户 ID"""
    is_notify: bool
    """是否通知"""
    is_whole_day: bool
    """是日期还是整点小时"""
    expire_time: str
    """事件发生的时间（毫秒级时间戳）"""
    notify_time: str
    """触发通知的时间（毫秒级时间戳）"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本局部样式"""


class InlineFile(BaseModel):
    file_token: str
    """附件 token"""
    source_block_id: str
    """当前文档中该文件所处的 block 的 ID"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本局部样式"""


class InlineBlock(BaseModel):
    block_id: str
    """关联的内联状态的 block 的 block_id"""
    text_element_style: Union[TextElementStyle, None] = None
    """文本局部样式"""


class Equation(BaseModel):
    content: str
    """符合 KaTeX 语法的公式内容，语法规则请参考：https://katex.org/docs/supported.html"""
    text_element_style: Union[TextElementStyle, None] = None


class TextElement(BaseModel):
    text_run: Union[TextRun, None] = None
    """文字。支持对 Page、Text、Heading1~9、Bullet、Ordered、Code、Quote、Todo 块进行修改"""
    mention_user: Union[MentionUser, None] = None
    """@用户。支持对 Text、Heading1~9、Bullet、Ordered、Quote、Todo 块进行修改。"""
    mention_doc: Union[MentionDoc, None] = None
    """@文档。支持对 Text、Heading1~9、Bullet、Ordered、Quote、Todo 块进行修改"""
    reminder: Union[Reminder, None] = None
    """日期提醒。支持对 Text、Heading1~9、Bullet、Ordered、Quote、Todo 块进行修改"""
    file: Union[InlineFile, None] = None
    """内联文件。仅支持删除或移动位置，不支持创建新的内联文件"""
    inline_block: Union[InlineBlock, None] = None
    """内联块。仅支持删除或移动位置，不支持创建新的内联块"""
    equation: Union[Equation, None] = None
    """公式"""


class Text(BaseModel):
    style: Union[TextStyle, None] = None
    """文本样式"""
    elements: Union[List[TextElement], None] = None
    """文本元素"""


class Bitable(BaseModel):
    token: str
    """多维表格 Token"""


class Callout(BaseModel):
    backgroud_color: Union[TextElementBackgroundColor, None] = None
    border_color: Union[BorderColor, None] = None
    text_color: Union[TextElementTextColor, None] = None
    emoji_id: Union[str, None] = None
    """todo"""


class ChatCard(BaseModel):
    chat_id: str
    align: Union[TextStyleAlign, None] = None


class Diagram(BaseModel):
    diagram_type: DiagramType


class Divider(BaseModel):
    pass


class File(BaseModel):
    token: str
    name: str
    view_type: ViewType


class Grid(BaseModel):
    column_size: int


class GridColumn(BaseModel):
    width_ratio: int


class IframeComponent(BaseModel):
    iframe_type: IframeType
    url: str


class Iframe(BaseModel):
    component: IframeComponent


class Image(BaseModel):
    width: int
    """图片宽度px"""
    height: int
    """图片高度px"""
    token: str
    """图片 Token"""
    align: Union[TextStyleAlign, None] = None


class ISVWidget(BaseModel):
    component_id: str
    """团队互动应用唯一ID"""
    component_type_id: str
    """团队互动应用类型，比如信息收集"blk_5f992038c64240015d280958" """


class AddOns(BaseModel):
    component_id: str
    """文档小组件 ID"""
    component_type_id: str
    """文档小组件类型，比如问答互动"blk_636a0a6657db8001c8df5488" """
    record: str
    """文档小组件内容数据，JSON 字符串"""


class MindNode(BaseModel):
    token: str
    """思维导图 Token"""


class Sheet(BaseModel):
    token: str
    """电子表格 Token"""


class TableMergeInfo(BaseModel):
    row_span: int
    """从当前行索引起被合并的连续行数"""
    col_span: int
    """从当前列索引起被合并的连续列数"""


class TableProperty(BaseModel):
    row_size: int
    """行数"""
    column_size: int
    """列数"""
    column_width: Union[List[int], None] = None
    """列宽，单位px"""
    merge_info: Union[List[TableMergeInfo]] = None
    """单元格合并信息。创建 Table 时，此属性只读，将由系统自动生成。如果需要合并单元格，可以通过更新块接口的子请求 merge_table_cells 实现"""
    header_row: Union[bool, None] = None
    """设置首行为标题行"""
    header_column: Union[bool, None] = None
    """设置首列为标题列"""


class Table(BaseModel):
    cells: List[str]
    """单元格数组，数组元素为 Table Cell Block 的 ID"""
    property: TableProperty
    """表格属性"""


class TableCell(BaseModel):
    pass


class Undefined(BaseModel):
    pass


class QuoteContainer(BaseModel):
    pass


class Task(BaseModel):
    task_id: str
    """任务 ID，查询具体任务详情见 https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/task-v2/task/get"""
    folded: bool


class OKRPeriodStatus(enum.Enum):
    DEFAULT = "default"
    NORMAL = "normal"
    INVALID = "invalid"
    HIDDEN = "hidden"


class OKRVisibleSetting(BaseModel):
    progress_fill_area_visible: bool
    """进展编辑区域是否可见"""
    progress_status_visible: bool
    """进展状态是否可见"""
    score_visible: bool
    """分数是否可见"""


class OKR(BaseModel):
    okr_id: str
    """OKR ID，获取需要插入的 OKR ID 可见"""
    period_display_status: OKRPeriodStatus
    period_name_zh: str
    """周期名 - 中文"""
    period_name_en: str
    """周期名 - 英文"""
    user_id: str
    """OKR 所属的用户 ID"""
    visible_setting: OKRVisibleSetting
    """OKR 可见性设置"""


class OKRProgressRate(BaseModel):
    mode: Literal["simple", "advanced"]
    current: Union[float, None] = None
    percent: Union[float, None] = None
    progress_status: Literal[
        "unset",
        "normal",
        "risk",
        "extended",
    ]
    start: Union[float, None] = None
    status_type: Literal["default", "custom"]
    target: Union[float, None] = None


class OKRObjective(BaseModel):
    objective_id: str
    """Objective ID"""
    confidential: bool
    """是否在 OKR 平台设置了私密权限"""
    position: int
    """是否在 OKR 平台设置了私密权限"""
    score: int
    """打分信息"""
    visible: bool
    """OKR Block 中是否展示该 Objective"""
    weight: float
    """Objective 的权重"""
    progress_rate: OKRProgressRate
    """进展信息"""
    content: Text
    """Objective 的文本内容"""


class OKRKeyResult(BaseModel):
    kr_id: str
    """Key Result 的 ID"""
    confidential: bool
    """是否在 OKR 平台设置了私密权限"""
    position: int
    """Key Result 的位置编号，对应 Block 中 KR1、KR2 的 1、2。"""
    score: int
    """打分信息"""
    visible: bool
    """OKR Block 中是否展示该 Key Result"""
    weight: float
    """Key Result 的权重"""
    progress_rate: OKRProgressRate
    """进展信息"""
    content: Text
    """Key Result 的文本内容"""


class OKRProgress(BaseModel):
    pass


class JiraIssue(BaseModel):
    id: str
    """Jira 问题 ID"""
    key: str
    """Jira 问题 key"""


class WikiCatalog(BaseModel):
    wiki_token: str
    """知识库 token"""


class Board(BaseModel):
    token: str
    """画板 Token"""
    align: Union[TextStyleAlign, None] = None
    width: Union[int, None] = None
    """宽度，单位 px；不填时自动适应文档宽度；值超出文档最大宽度时，页面渲染为文档最大宽度"""
    height: Union[int, None] = None
    """高度，单位 px；不填时自动适应内容高度；值超出文档最大高度时，页面渲染为文档最大高度"""


class Agenda(BaseModel):
    pass


class AgendaItem(BaseModel):
    pass


class AgendaItemTitleElement(BaseModel):
    text_run: Union[TextRun, None] = None
    mention_user: Union[MentionUser, None] = None
    mention_doc: Union[MentionDoc, None] = None
    reminder: Union[Reminder, None] = None
    file: Union[InlineFile, None] = None
    undefined: Union[Undefined, None] = None
    inline_block: Union[InlineBlock, None] = None
    equation: Union[Equation, None] = None


class AgendaItemContent(BaseModel):
    pass


class AgendaItemTitle(BaseModel):
    elements: List[AgendaItemTitleElement]
    align: Union[TextStyleAlign, None] = None
    agenda_item_content: AgendaItemContent


class LinkPreview(BaseModel):
    url_type: Literal["MessageLink", "Undefined"]


class BlockInfo(BaseModel):
    block_id: str
    """子块的唯一标识"""
    block_type: BlockType
    """Block 类型"""
    parent_id: Union[str, None] = None
    """子块的父块 ID"""
    children: Union[List[str], None] = None
    """子块的子块 ID 列表"""
    page: Union[Text, None] = None
    """文档的根 Block，也称页面 Block"""
    text: Union[Text, None] = None
    heading1: Union[Text, None] = None
    heading2: Union[Text, None] = None
    heading3: Union[Text, None] = None
    heading4: Union[Text, None] = None
    heading5: Union[Text, None] = None
    heading6: Union[Text, None] = None
    heading7: Union[Text, None] = None
    heading8: Union[Text, None] = None
    heading9: Union[Text, None] = None
    bullet: Union[Text, None] = None
    ordered: Union[Text, None] = None
    code: Union[Text, None] = None
    quote: Union[Text, None] = None
    equation: Union[Equation, None] = None
    todo: Union[Text, None] = None
    bitable: Union[Bitable, None] = None
    callout: Union[Callout, None] = None
    chat_card: Union[ChatCard, None] = None
    diagram: Union[Diagram, None] = None
    divider: Union[Divider, None] = None
    file: Union[File, None] = None
    grid: Union[Grid, None] = None
    grid_column: Union[GridColumn, None] = None
    iframe: Union[Iframe, None] = None
    image: Union[Image, None] = None
    isv: Union[ISVWidget, None] = None
    add_ons: Union[AddOns, None] = None
    mindnote: Union[MindNode, None] = None
    sheet: Union[Sheet, None] = None
    table: Union[Table, None] = None
    table_cell: Union[TableCell, None] = None
    view: Union[ViewType, None] = None
    undefined: Union[Undefined, None] = None
    quote_container: Union[QuoteContainer, None] = None
    task: Union[Task, None] = None
    okr: Union[OKR, None] = None
    okr_objective: Union[OKRObjective, None] = None
    okr_key_result: Union[OKRKeyResult, None] = None
    okr_progress: Union[OKRProgress, None] = None
    comment_ids: Union[List[str], None] = None
    jira_issue: Union[JiraIssue, None] = None
    wiki_catalog: Union[WikiCatalog, None] = None
    board: Union[Board, None] = None
    agenda: Union[Agenda, None] = None
    agenda_item: Union[AgendaItem, None] = None
    agenda_item_title: Union[AgendaItemTitle, None] = None
    link_preview: Union[LinkPreview, None] = None



from typing import List, Union

from ..base import BaseElement
from .button import ButtonElement
from .checker import CheckerElement
from .date_picker import DatePickerElement
from .input_box import InputBoxElement
from .multi_select_person import MultiSelectPersonElement
from .multi_select_static import MultiSelectStaticElement
from .overflow import OverflowElement
from .picker_datetime import PickerDatetimeElement
from .picker_time import PickerTimeElement
from .select_img import SelectImgElement
from .select_person import SelectPersonElement
from .select_static import SelectStaticElement

ActionElementType = Union[
    ButtonElement,
    CheckerElement,
    DatePickerElement,
    InputBoxElement,
    MultiSelectPersonElement,
    MultiSelectStaticElement,
    OverflowElement,
    PickerDatetimeElement,
    PickerTimeElement,
    SelectImgElement,
    SelectPersonElement,
    SelectStaticElement,
]


class ActionElements(BaseElement):
    tag: str = "action"
    layout: Union[str, None] = None
    actions: List[ActionElementType]



from typing import Dict, List, Union

import httpx
from typing_extensions import Literal

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.bitables.record.common import FieldValueType
from slark.types.bitables.record.request import (
    BatchCreateRecordBody,
    BatchDeleteRecordBody,
    BatchGetRecordBody,
    BatchUpdateRecord,
    BatchUpdateRecordBody,
    CreateRecordBody,
    CreateRecordParams,
    SearchRecordBody,
    SearchRecordFilter,
    SearchRecordParams,
    SearchRecordSort,
    UpdateRecordBody,
    UpdateRecordParams,
)
from slark.types.bitables.record.response import (
    BatchCreateRecordResponse,
    BatchDeleteRecordResponse,
    BatchGetRecordResponse,
    BatchUpdateRecordResponse,
    CreateRecordResponse,
    DeleteRecordResponse,
    SearchRecordResponse,
    UpdateRecordResponse,
)


class AsyncRecord(AsyncAPIResource):
    MAX_RECORDS_PER_REQUEST = 500

    async def create(
        self,
        app_token: str,
        *,
        table_id: str,
        fields: Dict[str, FieldValueType],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateRecordResponse:
        """该接口用于在数据表中新增一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/create

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            fields (Dict[str, FieldValueType]): 数据表的字段，即数据表的列。\
                [数据结构](https://open.feishu.cn/document/server-docs/docs/bitable-v1/bitable-structure)
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): \
                用户 ID 类型. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。\
                此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            CreateRecordResponse: 新增记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.create_record.format(app_token=app_token, table_id=table_id),
            body=CreateRecordBody(
                fields=fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateRecordParams(
                    user_id_type=user_id_type, client_token=client_token
                ).model_dump(),
            },
            cast_to=CreateRecordResponse,
        )

    async def update(
        self,
        app_token: str,
        *,
        table_id: str,
        record_id: str,
        fields: Dict[str, FieldValueType],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateRecordResponse:
        """该接口用于在数据表中更新一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/update

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_id (str): 一条记录的唯一标识 id 
            fields (Dict[str, FieldValueType]): 数据表的字段，即数据表的列。
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): \
                用户 ID 类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UpdateRecordResponse: 更新记录的返回结果
        """
        return await self._put(
            API_PATH.bitables.update_record.format(
                app_token=app_token, table_id=table_id, record_id=record_id
            ),
            body=UpdateRecordBody(
                fields=fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": UpdateRecordParams(user_id_type=user_id_type).model_dump(),
            },
            cast_to=UpdateRecordResponse,
        )

    async def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        record_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> DeleteRecordResponse:
        """该接口用于在数据表中删除一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/delete

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_id (str): 一条记录的唯一标识 id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            DeleteRecordResponse: 删除记录的返回结果
        """
        return await self._delete(
            API_PATH.bitables.delete_record.format(
                app_token=app_token, table_id=table_id, record_id=record_id
            ),
            options={"timeout": timeout},
            cast_to=DeleteRecordResponse,
        )

    async def search(
        self,
        app_token: str,
        *,
        table_id: str,
        view_id: Union[str, None] = None,
        field_names: Union[str, None] = None,
        sort: Union[List[SearchRecordSort], None] = None,
        filter: Union[SearchRecordFilter, None] = None,
        automatic_fields: Union[bool, None] = None,
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> SearchRecordResponse:
        """该接口用于查询数据表中的现有记录，单次最多查询 500 行记录，支持分页获取。
        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/search

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            view_id (Union[str, None], optional): 视图的唯一标识符，获取指定视图下的记录. Defaults to None.
            field_names (Union[str, None], optional): 字段名称，用于指定本次查询返回记录中包含的字段. Defaults to None.
            sort (Union[List[SearchRecordSort], None], optional): 排序条件. Defaults to None.
            filter (Union[SearchRecordFilter, None], optional): 筛选条件. Defaults to None.
            automatic_fields (Union[bool, None], optional): 控制是否返回自动计算的字段, true 表示返回. Defaults to None.
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果. Defaults to None.
            page_size (Union[int, None], optional): 分页大小。最大值为 500. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            SearchRecordResponse: 查询记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.search_record.format(app_token=app_token, table_id=table_id),
            body=SearchRecordBody(
                view_id=view_id,
                field_names=field_names,
                sort=sort,
                filter=filter,
                automatic_fields=automatic_fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": SearchRecordParams(
                    user_id_type=user_id_type,
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=SearchRecordResponse,
        )

    async def batch_get(
        self,
        app_token: str,
        *,
        table_id: str,
        record_ids: List[str],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        with_shared_url: Union[bool, None] = None,
        automatic_fields: Union[bool, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchGetRecordResponse:
        """该接口用于批量获取数据表中的记录
        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/batch_get

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_ids (List[str]): 记录的唯一标识 id 列表
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 此次调用中使用的用户 id 的类型. Defaults to None.
            with_shared_url (Union[bool, None], optional): 是否返回记录的分享链接。. Defaults to None.
            automatic_fields (Union[bool, None], optional): 是否返回自动计算的字段。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            BatchGetRecordResponse: 批量获取记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_get_record.format(app_token=app_token, table_id=table_id),
            body=BatchGetRecordBody(
                record_ids=record_ids,
                user_id_type=user_id_type,
                with_shared_url=with_shared_url,
                automatic_fields=automatic_fields,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=BatchGetRecordResponse,
        )

    async def batch_create(
        self,
        app_token: str,
        *,
        table_id: str,
        records: List[Dict[str, FieldValueType]],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchCreateRecordResponse:
        """该接口用于在数据表中新增多条记录，单次调用最多新增 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_create

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            records (List[Dict[str, FieldValueType]]): 记录的字段，即数据表的列
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchCreateRecordResponse: 批量新增记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_create_record.format(app_token=app_token, table_id=table_id),
            body=BatchCreateRecordBody(
                records=[{"fields": record} for record in records],
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateRecordParams(
                    user_id_type=user_id_type, client_token=client_token
                ).model_dump(),
            },
            cast_to=BatchCreateRecordResponse,
        )

    async def batch_update(
        self,
        app_token: str,
        *,
        table_id: str,
        records: List[BatchUpdateRecord],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchUpdateRecordResponse:
        """该接口用于更新数据表中的多条记录，单次调用最多更新 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_update

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            records (List[BatchUpdateRecord]): 记录的字段，即数据表的列
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户ID类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchUpdateRecordResponse: 更新记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_update_record.format(app_token=app_token, table_id=table_id),
            body=BatchUpdateRecordBody(records=records).model_dump(),
            options={
                "timeout": timeout,
                "params": UpdateRecordParams(user_id_type=user_id_type).model_dump(),
            },
            cast_to=BatchUpdateRecordResponse,
        )

    async def batch_delete(
        self,
        app_token: str,
        *,
        table_id: str,
        record_ids: List[str],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchDeleteRecordResponse:
        """该接口用于删除数据表中现有的多条记录，单次调用中最多删除 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_delete

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_ids (List[str]): 记录的唯一标识 id 列表
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchDeleteRecordResponse: 删除记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_delete_record.format(app_token=app_token, table_id=table_id),
            body=BatchDeleteRecordBody(
                records=record_ids,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=BatchDeleteRecordResponse,
        )


class Record(APIResource):
    MAX_RECORDS_PER_REQUEST = 500

    async def create(
        self,
        app_token: str,
        *,
        table_id: str,
        fields: Dict[str, FieldValueType],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateRecordResponse:
        """该接口用于在数据表中新增一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/create

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            fields (Dict[str, FieldValueType]): 数据表的字段，即数据表的列。\
                [数据结构](https://open.feishu.cn/document/server-docs/docs/bitable-v1/bitable-structure)
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): \
                用户 ID 类型. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。\
                此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            CreateRecordResponse: 新增记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.create_record.format(app_token=app_token, table_id=table_id),
            body=CreateRecordBody(
                fields=fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateRecordParams(
                    user_id_type=user_id_type, client_token=client_token
                ).model_dump(),
            },
            cast_to=CreateRecordResponse,
        )

    async def update(
        self,
        app_token: str,
        *,
        table_id: str,
        record_id: str,
        fields: Dict[str, FieldValueType],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateRecordResponse:
        """该接口用于在数据表中更新一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/update

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_id (str): 一条记录的唯一标识 id 
            fields (Dict[str, FieldValueType]): 数据表的字段，即数据表的列。
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): \
                用户 ID 类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UpdateRecordResponse: 更新记录的返回结果
        """
        return await self._put(
            API_PATH.bitables.update_record.format(
                app_token=app_token, table_id=table_id, record_id=record_id
            ),
            body=UpdateRecordBody(
                fields=fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": UpdateRecordParams(user_id_type=user_id_type).model_dump(),
            },
            cast_to=UpdateRecordResponse,
        )

    async def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        record_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> DeleteRecordResponse:
        """该接口用于在数据表中删除一条记录
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/delete

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_id (str): 一条记录的唯一标识 id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            DeleteRecordResponse: 删除记录的返回结果
        """
        return await self._delete(
            API_PATH.bitables.delete_record.format(
                app_token=app_token, table_id=table_id, record_id=record_id
            ),
            options={"timeout": timeout},
            cast_to=DeleteRecordResponse,
        )

    async def search(
        self,
        app_token: str,
        *,
        table_id: str,
        view_id: Union[str, None] = None,
        field_names: Union[str, None] = None,
        sort: Union[List[SearchRecordSort], None] = None,
        filter: Union[SearchRecordFilter, None] = None,
        automatic_fields: Union[bool, None] = None,
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> SearchRecordResponse:
        """该接口用于查询数据表中的现有记录，单次最多查询 500 行记录，支持分页获取。
        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/search

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            view_id (Union[str, None], optional): 视图的唯一标识符，获取指定视图下的记录. Defaults to None.
            field_names (Union[str, None], optional): 字段名称，用于指定本次查询返回记录中包含的字段. Defaults to None.
            sort (Union[List[SearchRecordSort], None], optional): 排序条件. Defaults to None.
            filter (Union[SearchRecordFilter, None], optional): 筛选条件. Defaults to None.
            automatic_fields (Union[bool, None], optional): 控制是否返回自动计算的字段, true 表示返回. Defaults to None.
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果. Defaults to None.
            page_size (Union[int, None], optional): 分页大小。最大值为 500. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            SearchRecordResponse: 查询记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.search_record.format(app_token=app_token, table_id=table_id),
            body=SearchRecordBody(
                view_id=view_id,
                field_names=field_names,
                sort=sort,
                filter=filter,
                automatic_fields=automatic_fields,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": SearchRecordParams(
                    user_id_type=user_id_type,
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=SearchRecordResponse,
        )

    async def batch_get(
        self,
        app_token: str,
        *,
        table_id: str,
        record_ids: List[str],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        with_shared_url: Union[bool, None] = None,
        automatic_fields: Union[bool, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchGetRecordResponse:
        """该接口用于批量获取数据表中的记录
        https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/bitable-v1/app-table-record/batch_get

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_ids (List[str]): 记录的唯一标识 id 列表
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 此次调用中使用的用户 id 的类型. Defaults to None.
            with_shared_url (Union[bool, None], optional): 是否返回记录的分享链接。. Defaults to None.
            automatic_fields (Union[bool, None], optional): 是否返回自动计算的字段。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): _description_. Defaults to None.

        Returns:
            BatchGetRecordResponse: 批量获取记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_get_record.format(app_token=app_token, table_id=table_id),
            body=BatchGetRecordBody(
                record_ids=record_ids,
                user_id_type=user_id_type,
                with_shared_url=with_shared_url,
                automatic_fields=automatic_fields,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=BatchGetRecordResponse,
        )

    async def batch_create(
        self,
        app_token: str,
        *,
        table_id: str,
        records: List[Dict[str, FieldValueType]],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchCreateRecordResponse:
        """该接口用于在数据表中新增多条记录，单次调用最多新增 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_create

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            records (List[Dict[str, FieldValueType]]): 记录的字段，即数据表的列
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchCreateRecordResponse: 批量新增记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_create_record.format(app_token=app_token, table_id=table_id),
            body=BatchCreateRecordBody(
                records=[{"fields": record} for record in records],
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateRecordParams(
                    user_id_type=user_id_type, client_token=client_token
                ).model_dump(),
            },
            cast_to=BatchCreateRecordResponse,
        )

    async def batch_update(
        self,
        app_token: str,
        *,
        table_id: str,
        records: List[BatchUpdateRecord],
        user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchUpdateRecordResponse:
        """该接口用于更新数据表中的多条记录，单次调用最多更新 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_update

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            records (List[BatchUpdateRecord]): 记录的字段，即数据表的列
            user_id_type (Union[Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], None], optional): 用户ID类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchUpdateRecordResponse: 更新记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_update_record.format(app_token=app_token, table_id=table_id),
            body=BatchUpdateRecordBody(records=records).model_dump(),
            options={
                "timeout": timeout,
                "params": UpdateRecordParams(user_id_type=user_id_type).model_dump(),
            },
            cast_to=BatchUpdateRecordResponse,
        )

    async def batch_delete(
        self,
        app_token: str,
        *,
        table_id: str,
        record_ids: List[str],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchDeleteRecordResponse:
        """该接口用于删除数据表中现有的多条记录，单次调用中最多删除 500 条记录。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-record/batch_delete

        Args:
            app_token (str): 多维表格的唯一标识符
            table_id (str): 多维表格数据表的唯一标识符
            record_ids (List[str]): 记录的唯一标识 id 列表
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchDeleteRecordResponse: 删除记录的返回结果
        """
        return await self._post(
            API_PATH.bitables.batch_delete_record.format(app_token=app_token, table_id=table_id),
            body=BatchDeleteRecordBody(
                records=record_ids,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=BatchDeleteRecordResponse,
        )



from typing import List, Union

from typing_extensions import Literal

from ..base import BaseElement
from ..behavior import BehaviorType
from ..confirm import ConfirmDialogue
from ..icon import IconElement
from ..text import PlainText

"""https://open.feishu.cn/document/uAjLw4CM/ukzMukzMukzM/feishu-cards/card-components/interactive-components/button"""

"""在卡片 JSON 1.0 结构中，若按钮组件直接位于卡片根节点，而非嵌套在其它组件中，你需将其 JSON 数据配置在交互模块（"tag": "action"）的 actions 字段中使用。

卡片 JSON 2.0 结构已不支持交互模块（"tag": "action"）相关属性。你可直接将按钮放置于 elements 中，并配置合适的组件间距 (vertical_spacing 和 horizontal_spacing) 使用。

嵌套规则

按钮组件支持嵌套在分栏、表单容器、折叠面板、循环容器中使用。
"""


class ButtonElement(BaseElement):
    tag: str = "button"
    type: Literal[
        "default",
        "primary",
        "danger",
        "text",
        "primary_text",
        "danger_text",
        "primary_filled",
        "danger_filled",
        "laser",
    ] = "default"
    """按钮的类型。可选值：

    default：黑色字体按钮，有边框
    primary：蓝色字体按钮，有边框
    danger：红色字体按钮，有边框
    text：黑色字体按钮，无边框
    primary_text：蓝色字体按钮，无边框
    danger_text：红色字体按钮，无边框
    primary_filled：蓝底白字按钮
    danger_filled：红底白字按钮
    laser：镭射按钮"""

    name: str
    """按钮的名称，用于标识按钮。"""
    form_action_type: Literal["submit", "reset", "none"] = "none"
    """按钮的行为。可选值"submit"、"reset"、"none"。"""
    size: Literal["tiny", "small", "medium", "large"] = "medium"
    """按钮的尺寸。可选值：

    tiny：超小尺寸，PC 端为 24 px；移动端为 28 px
    small：小尺寸，PC 端为 28 px；移动端为 28 px
    medium：中尺寸，PC 端为 32 px；移动端为 36 px
    large：大尺寸，PC 端为 40 px；移动端为 48 px"""

    width: Union[Literal["default", "fill"], str] = "default"
    """按钮的宽度。支持以下枚举值：

    default：默认宽度
    fill：卡片最大支持宽度
    [100,∞)px：自定义宽度，如 120px。超出卡片宽度时将按最大支持宽度展示"""

    text: Union[PlainText, None] = None
    """按钮上的文本。"""

    icon: Union[IconElement, None] = None
    """添加图标作为文本前缀图标。支持自定义或使用图标库中的图标。"""

    hover_tips: Union[PlainText, None] = None
    """用户在 PC 端将光标悬浮在交互容器上方时的文案提醒。默认为空。"""

    disabled: Union[bool, None] = None
    disabled_tips: Union[PlainText, None] = None
    """禁用按钮后，用户触发交互时的弹窗文案提醒。默认为空，即不弹窗。"""

    confirm: Union[ConfirmDialogue, None] = None
    """二次确认弹窗配置。指在用户提交时弹出二次确认弹窗提示；只有用户点击确认后，才提交输入的内容。该字段默认提供了确认和取消按钮，你只需要配置弹窗的标题与内容即可。"""

    behaviors: Union[List[BehaviorType], None] = None



from pydantic import BaseModel

from slark.types.response import BaseResponse


class DocumentDisplaySetting(BaseModel):
    show_authors: bool
    """文档信息中是否展示文档作者"""
    show_create_time: bool
    """文档信息中是否展示文档创建时间"""
    show_pv: bool
    """文档信息中是否展示文档访问次数"""
    show_uv: bool
    """文档信息中是否展示文档访问人数"""
    show_like_count: bool
    """文档信息中是否展示点赞总数"""
    show_comment_count: bool
    """文档信息中是否展示评论总数"""
    show_related_matters: bool
    """文档信息中是否展示关联事项。暂未支持"""


class DocumentCover(BaseModel):
    token: str
    offset_ratio_x: float
    offset_ratio_y: float


class DocumentInfo(BaseModel):
    document_id: str
    """文档的唯一标识。"""
    revision_id: int
    """文档版本 ID。起始值为 1"""
    title: str
    """文档标题"""
    display_setting: DocumentDisplaySetting
    """文档展示设置"""
    cover: DocumentCover
    """文档封面"""


class DocumentResponseData(BaseModel):
    document: DocumentInfo


class DocumentResponse(BaseResponse):
    data: DocumentResponseData


class GetRawContentResponseData(BaseModel):
    content: str
    """文档纯文本"""


class GetRawContentResponse(BaseResponse):
    data: GetRawContentResponseData



from typing import List

from slark.types._common import BaseModel
from slark.types.response import BaseResponse

from .common import CellTypes


class PrependDataResponseDataUpdates(BaseModel):
    spreadsheetToken: str
    """电子表格的 token"""
    updatedRange: str
    """插入数据的范围"""
    updatedRows: int
    """更新的行总数"""
    updatedColumns: int
    """更新的列总数"""
    updatedCells: int
    """更新的单元格总数"""
    revision: int
    """工作表的版本号。从 0 开始计数，更新一次版本号加一。"""


class PrependDataResponseData(BaseModel):
    spreadsheetToken: str
    """电子表格的 token"""
    tableRange: str
    """插入数据的范围"""
    revision: int
    """	工作表的版本号。从 0 开始计数，更新一次版本号加一。"""
    updates: PrependDataResponseDataUpdates
    """插入数据的范围、更新的行列总数等"""


class PrependDataResponse(BaseResponse):
    data: PrependDataResponseData


class AppendDataResponse(PrependDataResponse):
    pass


class ReadDataValueRange(BaseModel):
    majorDimension: str = "ROWS"
    """返回的 values 数组中数据的呈现维度。固定取值 ROWS，即数据为从左到右、从上到下的读取顺序。"""
    range: str
    """读取的范围。为空时表示查询范围没有数据。"""
    revision: int
    """工作表的版本号。从 0 开始计数，更新一次版本号加一。"""
    values: List[List[CellTypes]]
    """指定范围中的数据"""


class ReadSingleRangeResponseData(BaseModel):
    revision: int
    spreadsheetToken: str
    valueRange: ReadDataValueRange


class ReadSingleRangeResponse(BaseResponse):
    data: ReadSingleRangeResponseData


class ReadMultiRangeResponseData(BaseModel):
    revision: int
    spreadsheetToken: str
    valueRanges: List[ReadDataValueRange]


class ReadMultiRangeResponse(BaseResponse):
    data: ReadMultiRangeResponseData


class WriteSingleRangeResponseData(PrependDataResponseDataUpdates):
    pass


class WriteSingleRangeResponse(BaseResponse):
    data: WriteSingleRangeResponseData


class WriteMultiRangeResponseDataResponse(BaseModel):
    spreadsheetToken: str
    updatedRange: str
    updatedRows: str
    updatedColumns: str
    updatedCells: str


class WriteMultiRangeResponseData(BaseModel):
    revision: int
    spreadsheetToken: str
    responses: List[WriteMultiRangeResponseDataResponse]


class WriteMultiRangeResponse(BaseResponse):
    data: WriteMultiRangeResponseData



import json
from typing import Union

import anyio
import httpx
from typing_extensions import Literal

import slark.types.card as card
from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types._common import BaseModel
from slark.types._utils import cached_property
from slark.types.messages.request import (
    MESSAGE_TYPE,
    EditMessageBody,
    GetMessageParams,
    ReplyMessageBody,
    SendMessageBody,
    SendMessageParams,
)
from slark.types.messages.response import (
    BaseResponse,
    EditMessageResponse,
    GetMessageResponse,
    ReplyMessageResponse,
    SendMessageResponse,
)

from .image import AsyncImage, Image


class DownloadFileInfo(BaseModel):
    filepath: str
    filename: str
    filetype: str


class AsyncMessages(AsyncAPIResource):
    @cached_property
    def image(self) -> AsyncImage:
        return AsyncImage(self._client)

    async def send(
        self,
        receive_id_type: Literal["open_id", "user_id", "email", "chat_id"],
        receive_id: str,
        msg_type: MESSAGE_TYPE,
        content: str,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> SendMessageResponse:
        """发送消息

        Args:
            receive_id_type (Literal[&quot;open_id&quot;, &quot;user_id&quot;, &quot;email&quot;, &quot;chat_id&quot;]): 用户 ID 类型
                示例值："open_id"\
                可选值有：\n
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID\
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？\
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？\
                email：以用户的真实邮箱来标识用户。\
                chat_id：以群 ID 来标识群聊。了解更多：如何获取群 ID
            receive_id (str): 消息接收者的 ID，ID 类型与查询参数 receive_id_type 的取值一致。\
                注意事项：\n
                给用户发送消息时，用户需要在机器人的可用范围内。例如，你需要给企业全员发送消息，则需要将应用的可用范围设置为全体员工。 \
                给群组发送消息时，机器人需要在该群组中，且在群组内拥有发言权限。 \
                如果消息接收者为用户，推荐使用用户的 open_id。
            msg_type (MESSAGE_TYPE): 消息类型
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。\
                注意：\n
                JSON 字符串需进行转义。例如，换行符 \\n 转义后为 \\n。\
                文本消息请求体最大不能超过 150 KB。\
                卡片消息、富文本消息请求体最大不能超过 30 KB。\
                如果使用卡片模板（template_id）发送消息，实际大小也包含模板对应的卡片数据大小。\
                如果消息中包含样式标签，会使实际消息体长度大于您输入的请求体长度。\
                图片需要先上传图片，然后使用图片的 Key 发消息。\
                音频、视频、文件需要先上传文件，然后使用文件的 Key 发消息。注意不能使用云文档上传素材接口返回的 file_token。\
                了解不同类型的消息内容格式、使用限制，可参见发送消息内容。
            uuid (Union[str, None], optional): 自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
                注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
                示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204". Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            SendMessageResponse: 返回值
        """
        return await self._post(
            API_PATH.message.send,
            body=SendMessageBody(
                receive_id=receive_id,
                msg_type=msg_type,
                content=content,
                uuid=uuid,
            ).model_dump(),
            cast_to=SendMessageResponse,
            options={
                "params": SendMessageParams(
                    receive_id_type=receive_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )

    async def reply(
        self,
        message_id: str,
        msg_type: MESSAGE_TYPE,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> ReplyMessageResponse:
        """回复消息

        Args:
            message_id (str): 消息 ID
            msg_type (MESSAGE_TYPE): 消息类型
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。\
                注意：\n
                JSON 字符串需进行转义。例如，换行符 \\n 转义后为 \\n。\
                文本消息请求体最大不能超过 150 KB。\
                卡片消息、富文本消息请求体最大不能超过 30 KB。\
                如果使用卡片模板（template_id）发送消息，实际大小也包含模板对应的卡片数据大小。\
                如果消息中包含样式标签，会使实际消息体长度大于您输入的请求体长度。\
                图片需要先上传图片，然后使用图片的 Key 发消息。\
                音频、视频、文件需要先上传文件，然后使用文件的 Key 发消息。注意不能使用云文档上传素材接口返回的 file_token。\
                了解不同类型的消息内容格式、使用限制，可参见发送消息内容。
            reply_in_thread (bool, optional): 是否以话题形式回复。取值为 true 时将以话题形式回复。\
                注意：如果要回复的消息已经是话题形式的消息，则默认以话题形式进行回复. Defaults to False.
            uuid (Union[str, None], optional): 自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
                注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
                示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204". Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
            
        Returns:
            ReplyMessageResponse: 返回值
        """
        return await self._post(
            API_PATH.message.reply.format(message_id=message_id),
            body=ReplyMessageBody(
                content=content,
                msg_type=msg_type,
                reply_in_thread=reply_in_thread,
                uuid=uuid,
            ).model_dump(),
            cast_to=ReplyMessageResponse,
            options={
                "timeout": timeout,
            },
        )

    async def reply_text(
        self,
        message_id: str,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return await self.reply(
            message_id=message_id,
            msg_type="text",
            content=json.dumps({"text": content}, ensure_ascii=False),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    async def reply_markdown(
        self,
        message_id: str,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return await self.reply(
            message_id=message_id,
            msg_type="interactive",
            content=card.InteractiveCard(
                elements=[card.MDElement(content=content)]
            ).model_dump_json(exclude_none=True),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    async def reply_card(
        self,
        message_id: str,
        card: card.InteractiveCard,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return await self.reply(
            message_id=message_id,
            msg_type="interactive",
            content=card.model_dump_json(exclude_none=True),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    async def edit(
        self,
        message_id: str,
        msg_type: Literal["text", "post"],
        content: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> EditMessageResponse:
        """编辑消息调用该接口编辑已发送的消息内容，支持编辑文本、富文本消息。如需编辑卡片消息，请使用更新应用发送的消息卡片接口。\n
            前提条件\n
            应用需要开启机器人能力。\n
            编辑用户单聊内的消息时，用户需要在机器人的可用范围内。\n
            编辑群组内的消息时，机器人需要在该群组中，且拥有发言权限。\n
            使用限制\n
            一条消息最多可编辑 20 次。\n
            仅可编辑当前操作者自己发送的消息。\n
            不可编辑已撤回，已删除，超出可编辑时间的消息。可编辑时间由企业管理员设定，详情了解管理员设置撤回和编辑消息权限。


        Args:
            message_id (str): message ID
            msg_type (Literal[&quot;text&quot;, &quot;post&quot;]): 消息类型。\n
                可选值有：\n
                text：文本\n
                post：富文本\n
                示例值："text"
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            EditMessageResponse: 返回值
        """
        return await self._put(
            API_PATH.message.edit.format(message_id=message_id),
            body=EditMessageBody(msg_type=msg_type, content=content).model_dump(),
            cast_to=EditMessageResponse,
            options={"timeout": timeout},
        )

    async def edit_text(
        self,
        message_id: str,
        content: str,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return await self.edit(
            message_id=message_id,
            msg_type="text",
            content=json.dumps({"text": content}, ensure_ascii=False),
            timeout=timeout,
        )

    async def edit_card(
        self,
        message_id: str,
        card: card.InteractiveCard,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        """[更新卡片消息](https://open.feishu.cn/document/server-docs/im-v1/message-card/patch)\
            若以 user_access_token 更新消息，该操作用户必须是卡片消息的发送者。\
            仅支持更新未撤回的共享卡片消息。你需在更新前后卡片的 config 属性中，均显式声明 "update_multi":true。如果更新前后的卡片存在非共享卡片，可能导致更新异常。

        Args:
            message_id (str): message ID
            card (card.InteractiveCard): 卡片消息内容
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 返回值
        """
        return await self._patch(
            API_PATH.message.edit.format(message_id=message_id),
            body={
                "content": card.model_dump_json(exclude_none=True),
            },
            cast_to=BaseResponse,
            options={"timeout": timeout},
        )

    async def get(
        self,
        message_id: str,
        user_id_type: Literal["open_id", "union_id", "user_id"] = "open_id",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetMessageResponse:
        return await self._get(
            API_PATH.message.get.format(message_id=message_id),
            cast_to=GetMessageResponse,
            options={
                "params": GetMessageParams(user_id_type=user_id_type).model_dump(),
                "timeout": timeout,
            },
        )

    async def get_resource(
        self,
        message_id: str,
        file_key: str,
        type: Literal["image", "file"],
        save_dir: str,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        response = await self._get(
            API_PATH.message.get_resource.format(message_id=message_id, file_key=file_key),
            options={
                "timeout": timeout,
                "raw_response": True,
                "params": {"type": type},
            },
            cast_to=httpx.Response,
        )
        mime_type = response.headers["Content-Type"]
        filetype, ext = mime_type.split("/")
        filename = f"{filetype}_{message_id}_{file_key}_{type}.{ext}"
        path = anyio.Path(save_dir) / filename
        await path.parent.mkdir(parents=True, exist_ok=True)
        f = await anyio.open_file(path, "wb")
        await f.write(response.content)
        await f.aclose()
        return DownloadFileInfo(
            filepath=path.as_posix(),
            filename=filename,
            filetype=mime_type,
        )


class Messages(APIResource):
    @cached_property
    def image(self) -> Image:
        return Image(self._client)

    def send(
        self,
        receive_id_type: Literal["open_id", "user_id", "email", "chat_id"],
        receive_id: str,
        msg_type: MESSAGE_TYPE,
        content: str,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> SendMessageResponse:
        """发送消息

        Args:
            receive_id_type (Literal[&quot;open_id&quot;, &quot;user_id&quot;, &quot;email&quot;, &quot;chat_id&quot;]): 用户 ID 类型
                示例值："open_id"\
                可选值有：\n
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID\
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？\
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？\
                email：以用户的真实邮箱来标识用户。\
                chat_id：以群 ID 来标识群聊。了解更多：如何获取群 ID
            receive_id (str): 消息接收者的 ID，ID 类型与查询参数 receive_id_type 的取值一致。\
                注意事项：\n
                给用户发送消息时，用户需要在机器人的可用范围内。例如，你需要给企业全员发送消息，则需要将应用的可用范围设置为全体员工。 \
                给群组发送消息时，机器人需要在该群组中，且在群组内拥有发言权限。 \
                如果消息接收者为用户，推荐使用用户的 open_id。
            msg_type (MESSAGE_TYPE): 消息类型
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。\
                注意：\n
                JSON 字符串需进行转义。例如，换行符 \\n 转义后为 \\n。\
                文本消息请求体最大不能超过 150 KB。\
                卡片消息、富文本消息请求体最大不能超过 30 KB。\
                如果使用卡片模板（template_id）发送消息，实际大小也包含模板对应的卡片数据大小。\
                如果消息中包含样式标签，会使实际消息体长度大于您输入的请求体长度。\
                图片需要先上传图片，然后使用图片的 Key 发消息。\
                音频、视频、文件需要先上传文件，然后使用文件的 Key 发消息。注意不能使用云文档上传素材接口返回的 file_token。\
                了解不同类型的消息内容格式、使用限制，可参见发送消息内容。
            uuid (Union[str, None], optional): 自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
                注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
                示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204". Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            SendMessageResponse: 返回值
        """
        return self._post(
            API_PATH.message.send,
            body=SendMessageBody(
                receive_id=receive_id,
                msg_type=msg_type,
                content=content,
                uuid=uuid,
            ).model_dump(),
            cast_to=SendMessageResponse,
            options={
                "params": SendMessageParams(
                    receive_id_type=receive_id_type,
                ).model_dump(),
                "timeout": timeout,
            },
        )

    def reply(
        self,
        message_id: str,
        msg_type: MESSAGE_TYPE,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> ReplyMessageResponse:
        """回复消息

        Args:
            message_id (str): 消息 ID
            msg_type (MESSAGE_TYPE): 消息类型
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。\
                注意：\n
                JSON 字符串需进行转义。例如，换行符 \\n 转义后为 \\n。\
                文本消息请求体最大不能超过 150 KB。\
                卡片消息、富文本消息请求体最大不能超过 30 KB。\
                如果使用卡片模板（template_id）发送消息，实际大小也包含模板对应的卡片数据大小。\
                如果消息中包含样式标签，会使实际消息体长度大于您输入的请求体长度。\
                图片需要先上传图片，然后使用图片的 Key 发消息。\
                音频、视频、文件需要先上传文件，然后使用文件的 Key 发消息。注意不能使用云文档上传素材接口返回的 file_token。\
                了解不同类型的消息内容格式、使用限制，可参见发送消息内容。
            reply_in_thread (bool, optional): 是否以话题形式回复。取值为 true 时将以话题形式回复。\
                注意：如果要回复的消息已经是话题形式的消息，则默认以话题形式进行回复. Defaults to False.
            uuid (Union[str, None], optional): 自定义设置的唯一字符串序列，用于在回复消息时请求去重。不填则表示不去重。持有相同 uuid 的请求，在 1 小时内至多成功回复一条消息。\
                注意：你可以参考示例值自定义参数值。当回复的内容不同时，如果传入了该参数，则需要在每次请求时都更换该参数的取值。\
                示例值："选填，每次调用前请更换，如a0d69e20-1dd1-458b-k525-dfeca4015204". Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
            
        Returns:
            ReplyMessageResponse: 返回值
        """
        return self._post(
            API_PATH.message.reply.format(message_id=message_id),
            body=ReplyMessageBody(
                content=content,
                msg_type=msg_type,
                reply_in_thread=reply_in_thread,
                uuid=uuid,
            ).model_dump(),
            cast_to=ReplyMessageResponse,
            options={
                "timeout": timeout,
            },
        )

    def reply_text(
        self,
        message_id: str,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return self.reply(
            message_id=message_id,
            msg_type="text",
            content=json.dumps({"text": content}, ensure_ascii=False),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    def reply_markdown(
        self,
        message_id: str,
        content: str,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return self.reply(
            message_id=message_id,
            msg_type="interactive",
            content=card.InteractiveCard(
                elements=[card.MDElement(content=content)]
            ).model_dump_json(exclude_none=True),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    def reply_card(
        self,
        message_id: str,
        card: card.InteractiveCard,
        reply_in_thread: bool = False,
        uuid: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return self.reply(
            message_id=message_id,
            msg_type="interactive",
            content=card.model_dump_json(exclude_none=True),
            reply_in_thread=reply_in_thread,
            uuid=uuid,
            timeout=timeout,
        )

    def edit(
        self,
        message_id: str,
        msg_type: Literal["text", "post"],
        content: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> EditMessageResponse:
        """编辑消息调用该接口编辑已发送的消息内容，支持编辑文本、富文本消息。如需编辑卡片消息，请使用更新应用发送的消息卡片接口。\n
            前提条件\n
            应用需要开启机器人能力。\n
            编辑用户单聊内的消息时，用户需要在机器人的可用范围内。\n
            编辑群组内的消息时，机器人需要在该群组中，且拥有发言权限。\n
            使用限制\n
            一条消息最多可编辑 20 次。\n
            仅可编辑当前操作者自己发送的消息。\n
            不可编辑已撤回，已删除，超出可编辑时间的消息。可编辑时间由企业管理员设定，详情了解管理员设置撤回和编辑消息权限。


        Args:
            message_id (str): message ID
            msg_type (Literal[&quot;text&quot;, &quot;post&quot;]): 消息类型。\n
                可选值有：\n
                text：文本\n
                post：富文本\n
                示例值："text"
            content (str): 消息内容，JSON 结构序列化后的字符串。该参数的取值与 msg_type 对应，例如 msg_type 取值为 text，则该参数需要传入文本类型的内容。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            EditMessageResponse: 返回值
        """
        return self._put(
            API_PATH.message.edit.format(message_id=message_id),
            body=EditMessageBody(msg_type=msg_type, content=content).model_dump(),
            cast_to=EditMessageResponse,
            options={"timeout": timeout},
        )

    def edit_text(
        self,
        message_id: str,
        content: str,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        return self.edit(
            message_id=message_id,
            msg_type="text",
            content=json.dumps({"text": content}, ensure_ascii=False),
            timeout=timeout,
        )

    def edit_card(
        self,
        message_id: str,
        card: card.InteractiveCard,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        """[更新卡片消息](https://open.feishu.cn/document/server-docs/im-v1/message-card/patch)\
            若以 user_access_token 更新消息，该操作用户必须是卡片消息的发送者。\
            仅支持更新未撤回的共享卡片消息。你需在更新前后卡片的 config 属性中，均显式声明 "update_multi":true。如果更新前后的卡片存在非共享卡片，可能导致更新异常。

        Args:
            message_id (str): message ID
            card (card.InteractiveCard): 卡片消息内容
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 返回值
        """
        return self._patch(
            API_PATH.message.edit.format(message_id=message_id),
            body={
                "content": card.model_dump_json(exclude_none=True),
            },
            cast_to=BaseResponse,
            options={"timeout": timeout},
        )

    def get(
        self,
        message_id: str,
        user_id_type: Literal["open_id", "union_id", "user_id"] = "open_id",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetMessageResponse:
        return self._get(
            API_PATH.message.get.format(message_id=message_id),
            cast_to=GetMessageResponse,
            options={
                "params": GetMessageParams(user_id_type=user_id_type).model_dump(),
                "timeout": timeout,
            },
        )

    def get_resource(
        self,
        message_id: str,
        file_key: str,
        type: Literal["image", "file"],
        save_dir: str,
        timeout: Union[httpx.Timeout, None] = None,
    ):
        response = self._get(
            API_PATH.message.get_resource.format(message_id=message_id, file_key=file_key),
            options={
                "timeout": timeout,
                "raw_response": True,
                "params": {"type": type},
            },
            cast_to=httpx.Response,
        )
        mime_type = response.headers["Content-Type"]
        filetype, ext = mime_type.split("/")
        filename = f"{filetype}_{message_id}_{file_key}_{type}.{ext}"
        path = anyio.Path(save_dir) / filename
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(path, "wb") as f:
            f.write(response.content)
        return DownloadFileInfo(
            filepath=path.as_posix(),
            filename=filename,
            filetype=mime_type,
        )




from typing import Dict, Union

from typing_extensions import Literal

from ..base import BaseElement


class ChartElement(BaseElement):
    tag: str = "chart"
    aspect_ratio: Union[Literal["1:1", "2:1", "4:3", "16:9"], None] = None
    color_theme: Union[
        Literal["brand", "rainbow", "complementary", "converse", "primary"], None
    ] = None
    """图表的主题样式。当图表内存在多个颜色时，可使用该字段调整颜色样式。若你在 chart_spec 字段中声明了样式类属性，该字段无效。\n
    brand：默认样式，与飞书客户端主题样式一致。
    rainbow：同色系彩虹色。
    complementary：互补色。
    converse：反差色。
    primary：主色。"""

    chart_spec: Dict
    """基于 VChart 的图表定义。详细用法参考 VChart 官方文档。\n
    提示：\n
    在飞书 7.1 - 7.6 版本上，图表组件支持的 VChart 版本为 1.2.2；
    在飞书 7.7 - 7.9 版本上，图表组件支持的 VChart 版本为 1.6.6；
    在飞书 7.10 - 7.15 版本上，图表组件支持的 VChart 版本为 1.8.3；
    在飞书 7.16 及以上版本上，图表组件支持的 VChart 版本为 1.10.1。了解 VChart 版本更新，参考 VChart Changelogs。"""

    preview: Union[bool, None] = None
    height: Union[str, None] = None


import enum
from typing import List, Union

from ..._common import BaseModel
from ..ud_icon import UDIconElement


class TextTagColor(enum.Enum):
    NEUTRAL = "neutral"
    BLUE = "blue"
    TURQUOISE = "turquoise"
    LIME = "lime"
    ORANGE = "orange"
    VIOLET = "violet"
    INDIGO = "indigo"
    WATHET = "wathet"
    GREEN = "green"
    YELLOW = "yellow"
    RED = "red"
    PURPLE = "purple"
    CARMINE = "carmine"


class CardTemplate(enum.Enum):
    BLUE = "blue"
    WATHET = "wath"
    TURQUOISE = "tuiquoise"
    GREEN = "green"
    YELLOW = "yellow"
    ORANGE = "orange"
    RED = "red"
    CARMINE = "carmine"
    VIOLET = "violet"
    PURPLE = "purple"
    INDIGO = "indigo"
    GREY = "grey"
    DEFAULT = "default"


class I18nText(BaseModel):
    zh_cn: Union[str, None] = None
    en_us: Union[str, None] = None
    ja_jp: Union[str, None] = None
    zh_hk: Union[str, None] = None
    zh_tw: Union[str, None] = None
    id_id: Union[str, None] = None
    vi_vn: Union[str, None] = None
    th_th: Union[str, None] = None
    pt_br: Union[str, None] = None
    es_es: Union[str, None] = None
    ko_kr: Union[str, None] = None
    de_de: Union[str, None] = None
    fr_fr: Union[str, None] = None
    it_it: Union[str, None] = None
    ru_ru: Union[str, None] = None
    ms_my: Union[str, None] = None


class CardHeaderTitle(BaseModel):
    tag: str = "plain_text"
    """固定值 plain_text。"""
    content: str
    """主标题内容。"""
    i18n: Union[I18nText, None] = None
    """多语言标题内容。必须配置 content 或 i18n 两个属性的其中一个。如果同时配置仅生效 i18n。"""


class CardHeaderSubtitle(BaseModel):
    tag: str = "plain_text"
    """固定值 plain_text。"""
    content: str
    """副标题内容。"""
    i18n: Union[I18nText, None] = None
    """多语言副标题内容。必须配置 content 或 i18n 两个属性的其中一个。如果同时配置仅生效 i18n。"""


class CardHeaderTextTagText(BaseModel):
    tag: str = "plain_text"
    content: str


class CardHeaderTextTag(BaseModel):
    tag: str = "text_tag"
    text: CardHeaderTextTagText
    """标签内容"""
    color: TextTagColor = TextTagColor.NEUTRAL.value
    """标签颜色"""


class CardHeaderI18nTextTag(BaseModel):
    zh_cn: Union[List[CardHeaderTextTag], None] = None
    en_us: Union[List[CardHeaderTextTag], None] = None
    ja_jp: Union[List[CardHeaderTextTag], None] = None
    zh_hk: Union[List[CardHeaderTextTag], None] = None
    zh_tw: Union[List[CardHeaderTextTag], None] = None
    id_id: Union[List[CardHeaderTextTag], None] = None
    vi_vn: Union[List[CardHeaderTextTag], None] = None
    th_th: Union[List[CardHeaderTextTag], None] = None
    pt_br: Union[List[CardHeaderTextTag], None] = None
    es_es: Union[List[CardHeaderTextTag], None] = None
    ko_kr: Union[List[CardHeaderTextTag], None] = None
    de_de: Union[List[CardHeaderTextTag], None] = None
    fr_fr: Union[List[CardHeaderTextTag], None] = None
    it_it: Union[List[CardHeaderTextTag], None] = None
    ru_ru: Union[List[CardHeaderTextTag], None] = None
    ms_my: Union[List[CardHeaderTextTag], None] = None


class CardHeaderIcon(BaseModel):
    img_key: str
    """用作前缀图标的图片 key。"""


class CardHeader(BaseModel):
    title: CardHeaderTitle
    """卡片主标题。必填。"""
    subtitle: Union[CardHeaderSubtitle, None] = None
    """卡片副标题。可选。"""
    text_tag_list: Union[List[CardHeaderTextTag], None] = None
    """标题后缀标签，最多设置 3 个 标签，超出不展示。可选。"""
    i18n_text_tag_list: Union[CardHeaderI18nTextTag, None] = None
    """国际化标题后缀标签。每个语言环境最多设置 3 个 tag，超出不展示。可选。同时配置原字段和国际化字段，优先生效国际化配置。"""
    template: CardTemplate = CardTemplate.DEFAULT.value
    """标题主题颜色。"""
    icon: Union[CardHeaderIcon, None] = None
    """自定义前缀图标"""
    ud_icon: Union[UDIconElement, None] = None
    """图标库中的前缀图标，和 icon 同时设置时以 ud_icon 为准。"""



from typing import List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel
from slark.types.response import BaseResponse


class GetSpreadsheetInfoResponseDataSpreadsheet(BaseModel):
    title: str
    """电子表格标题"""
    owner_id: str
    """电子表格的所有者 ID。ID 类型由查询参数 user_id_type 决定。"""
    token: str
    """电子表格 token"""
    url: str
    """电子表格的 URL 链接"""


class GetSpreadsheetInfoResponseData(BaseModel):
    spreadsheet: GetSpreadsheetInfoResponseDataSpreadsheet


class GetSpreadsheetInfoResponse(BaseResponse):
    data: GetSpreadsheetInfoResponseData


class WorksheetGridProperties(BaseModel):
    frozen_row_count: int
    frozen_column_count: int
    row_count: int
    column_count: int


class WorksheetMerge(BaseModel):
    start_row_index: int
    end_row_index: int
    start_column_index: int
    end_column_index: int


class WorksheetInfo(BaseModel):
    sheet_id: str
    """工作表 ID"""
    title: str
    """	工作表标题"""
    index: int
    """工作表索引位置，索引从 0 开始计数。"""
    hidden: bool
    """工作表是否被隐藏"""
    grid_properties: Union[WorksheetGridProperties, None] = None
    """单元格属性，仅当 resource_type 为 sheet 即工作表类型为电子表格时返回。"""
    resource_type: Literal["sheet", "bitable", "#UNSUPPORTED_TYPE"]
    """工作表类型，sheet 为工作表，bitable 为多维表格，#UNSUPPORTED_TYPE 为不支持的类型。"""
    merges: Union[List[WorksheetMerge], None] = None
    """合并单元格的相关信息。没有合并单元格则不返回。"""


class GetWorksheetInfoResponseData(BaseModel):
    sheet: WorksheetInfo


class GetWorksheetInfoResponse(BaseResponse):
    data: GetWorksheetInfoResponseData


class GetAllWorksheetsInfoResponseData(BaseModel):
    sheets: List[WorksheetInfo]


class GetAllWorksheetsInfoResponse(BaseResponse):
    data: GetAllWorksheetsInfoResponseData



from .callback import CallbackAction, CallbackContext, CallbackEvent, CallbackOperator
from .common import EventType, LarkEvent, LarkEventHeader
from .message import MessageEvent
from .response import CallbackCard, CallbackResponse, CallbackToast

__all__ = [
    "LarkEvent",
    "LarkEventHeader",
    "MessageEvent",
    "CallbackAction",
    "CallbackContext",
    "CallbackEvent",
    "CallbackOperator",
    "CallbackCard",
    "CallbackResponse",
    "EventType",
    "CallbackToast",
]
